import subprocess, os, zq_log

def git_gui_command(command:str, path:list, extCommand:str='', is_wait=True):
    '''
    command: "add"; "commit"; "update"; "revert";log
    '''
    path = [i.replace('\\','/') for i in path]
    all_path = '*'.join(path)
    command_string = "cmd /c TortoiseGitProc.exe /command:%s /path:\"%s\" %s /closeonend 2"%(command, all_path, extCommand)
    zq_log.log_debug('git: %s'%command_string)
    if is_wait:
        subprocess.Popen(command_string, stdin=subprocess.PIPE).wait()
    else:
        os.system(command_string)



import os, requests, json, time
import zq_file, zq_const, zq_log

class work_wx_class():
    def __init__(self,
        corpid = 'wwb6ec4438ccc388c3',
        secret = '_surPYo0SLVcxPKzAnHdWBwpGl2woxWKWpSTMvbZvnw',
        agentid = '1000002'
    ):
        self.corpid = corpid            # Corpid是企业号的标识
        self.secret = secret            # Secret是管理组凭证密钥
        self.agentid = agentid          # 应用ID
        self.bot = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=c2ae20ef-9a26-4791-abb5-962e07f97144'

    #https://www.runoob.com/markdown/md-block.html
    def work_wx_send_desc(self, desc='', userId=None):
        data={'msgtype':'text', 'text':{'content':desc}}
        if userId:
            data['touser'] = userId
        self.work_wx_send(data)
    
    def work_wx_get_token_from_server(self, corpid, secret, token_path):
        ''' 获取 token '''
        token = False
        try:
            if os.path.exists(token_path):
                file = open(token_path, 'r')
                jsonFileData = json.load(file)
                token = jsonFileData['access_token']
                expires_in = jsonFileData['expires_in']
                zq_file.close()
                
                modifyTime = os.path.getmtime(token_path)
                offtime = time.time() - modifyTime - expires_in
                if offtime > 0:
                    token = False
                    raise Exception('%s 校验文件已过期,需要重新获取校验信息'%token_path)
            else:
                zq_file.dir_auto_create_dir_by_file(token_path)
                raise Exception('%s 校验文件不存在需要创建'%token_path)
        except Exception as e:
            zq_log.log_debug('Error: %s'%e)
            url = 'https://qyapi.weixin.qq.com/cgi-bin/gettoken'
            data = {
                'corpid': corpid,
                'corpsecret': secret
            }
            r = requests.post(url=url, params=data, verify=False)
            zq_log.log_debug('企业微信Token: %s'%r.json())
            if r.json()['errcode'] != 0:
                token = False
            else:
                token = r.json()['access_token']
                file = open(token_path, 'w')
                file.write(r.text)
                file.close()
                zq_log.log_debug('%s 校验文件已更新'%token_path)
        return token

    def work_wx_send(self, data, corpid=None, secret=None, errorNum=3):
        ''' 
        发送消息到企业微信
        API文档： https://work.weixin.qq.com/api/doc/90000/90135/90236
        '''
        if not corpid or not secret:
            corpid = self.corpid
            secret = self.secret
        token_path = zq_const.const_merage_caches_dir('./work_wx/%s_wechat_config.json'%(corpid))
        # 获取token信息
        token = self.work_wx_get_token_from_server(corpid, secret, token_path)
        # 发送消息
        if not 'agentid' in data:
            data['agentid'] = self.agentid
        if not 'touser' in data:
            data['touser'] = '@all'
        if not 'toparty' in data:
            data['toparty'] = '1'
        url = 'https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token=%s' % token
        result = requests.post(url=url, data=json.dumps(data), verify=False)
        zq_log.log_debug('企业微信应用消息： %s'%result.json())
        # 如果发送失败，将重试多次if
        if result.json()['errcode'] != 0 and errorNum and errorNum > 0:
            if result.json()['errcode'] == 42001:
                os.remove(token_path)
                token = self.work_wx_get_token_from_server(corpid, secret, token_path)
            return self.work_wx_send(data, corpid, secret, errorNum-1)
        else:
            return result.json()
    
    def work_wx_send_bot(self, data, url=None):
        '''
        发送企业微信消息，机器人群
        API文档： https://work.weixin.qq.com/api/doc/90000/90136/91770
        '''
        url = url or self.bot
        if not 'mentioned_list' in data['text']:
            data['text']['mentioned_list'] = ['@all']
        if not 'mentioned_mobile_list' in data['text']:
            data['text']['mentioned_mobile_list'] = ['@all']
        zq_log.log_debug(json.dumps(data))
        result = requests.post(url=url, data=json.dumps(data), verify=False)
        zq_log.log_debug('企业微信应用消息： %s'%result.json())

if __name__ == "__main__":
    print(work_wx_class().work_wx_send_desc('99999'))


import os
import requests
import chardet
from contextlib import closing
import zq_log, zq_file

headers = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
    'accept-language': 'zh-CN,zh;q=0.9',
    # 'accept-encoding': 'gzip, deflate, br', #加密
    'pragma': 'no-cache',
    'cache-control': 'no-cache',
    'upgrade-insecure-requests': '1',
    # 'user-agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1'
    'user-agent': 'Mozilla/5.0 (Linux; U; Android 5.1.1; zh-cn; MI 4S Build/LMY47V) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/53.0.2785.146 Mobile Safari/537.36 XiaoMi/MiuiBrowser/9.1.3',
}

def http_get_requests_url_result(url, headers = None, params = None, data = None, stream = False, verify=False, timeout=60, proxies=False, isAutoEncoding=False):
    '''
    requests get 网络数据
    :param proxies: {"https":"127.0.0.1:12639"}
    '''
    requests.adapters.DEFAULT_RETRIES = 5
    s = requests.session()
    s.keep_alive = False
    res = None
    
    try:
        if headers:
            for headerItem in headers:
                headers[headerItem] = headers[headerItem]
        requests.packages.urllib3.disable_warnings()
        res = s.get(url, headers = headers, params = params, data = data, verify=verify, stream=stream, timeout=timeout, proxies=proxies)
        if isAutoEncoding:
            res.encoding = chardet.detect(res.content).get('encoding')
        res.close()            
        return res
    except BaseException as e:            
        zq_log.log_debug("http_get_requests_url_result BaseException error ==>> URL: {}  Error: {}".format(url, str(e)))        
    return res

def http_post_requests_url_result(url, headers = None, params = None, data = None, stream = False, files= False, verify=False, cookies=False, timeout=60, proxies=None, isAutoEncoding=False):
    '''
    requests post 网络数据
    :param proxies: {"https":"127.0.0.1:12639"}
    '''
    requests.adapters.DEFAULT_RETRIES = 5
    s = requests.session()
    s.keep_alive = False
    res = None
    
    try:
        if headers:
            for headerItem in headers:
                headers[headerItem] = headers[headerItem]
        requests.packages.urllib3.disable_warnings()
        res = s.post(url, headers = headers, params = params, data = data, verify=verify, stream=stream, files= files, cookies=cookies, timeout=timeout, proxies=proxies)
        if isAutoEncoding:
            res.encoding = chardet.detect(res.content).get('encoding')
        res.close()            
        return res
    except BaseException as e:            
        zq_log.log_debug("http_post_requests_url_result BaseException error ==>> URL: {}  Error: {}".format(url, str(e)))        
    return res

def http_download(urlPath, saveFilePath, exists_jump=True, desc='文件下载中: '):
    '''
    http下载  适用于大文件
    :param urlPath: 下载地址
    :param saveFilePath: 保存路径，带后缀
    :param exists_jump: 已存在是否跳过
    :return bool: 是否成功
    '''
    try:
        requests.adapters.DEFAULT_RETRIES = 5
        s = requests.session()
        s.keep_alive = False
        with closing(requests.get(urlPath, headers = headers, stream=True)) as response:
            if not response.ok:
                zq_log.log_debug('错误页面：==>>>  ' + response.text)
                return False
            chunk_size = 1024
            content_size = -1
            if 'content-length' in response.headers:
                content_size = int(response.headers['content-length'])
            if(os.path.exists(saveFilePath) and os.path.getsize(saveFilePath)==content_size) and exists_jump:
                zq_log.log_debug('跳过'+saveFilePath)
                return True
            else:
                videoDirPath = os.path.dirname(saveFilePath)
                zq_file.dir_auto_create_dir_by_file(saveFilePath)
                with open(saveFilePath, "wb") as fs:
                    curr_size = 0
                    for data in response.iter_content(chunk_size=chunk_size):
                        fs.write(data)
                        curr_size += len(data)
                        zq_log.log_progress_bar(curr_size, content_size, desc)
        return True
    except BaseException as e:
        zq_log.log_debug("http_download error ==>>  " + str(e))
        return False

def http_download_img(urlPath, saveFilePath):
    '''
    http下载  适用于小文件，图片那种，很快的
    '''
    from urllib.request import urlretrieve
    return urlretrieve(urlPath, filename=saveFilePath)


import os, sys, re
import zq_log

dir_path = os.path.dirname(__file__).replace('\\','/')
dir_name = os.path.basename(dir_path)
for _root, _dirs, _files in os.walk(dir_path):
    sys.path.append(_root)
    imports = []
    # 文件形式引入
    for _file in _files:
        if not _file == '__init__.py' and _file.endswith('.py'):
            _file = _file.replace('.py','')
            imports.append(_file)
    # 文件夹形式引入
    for _dir in _dirs:
        if not _dir in ['__pycache__']:
            imports.append(_dir)

    for item in imports:
        zq_log.log_debug('cv模块下的内容不常用，需要单独引入 -> import zqpy.%s.%s'%(dir_name, item))
        # exec('import %s'%(item))
        # exec('if hasattr(%s, "docs"): %s.docs()'%(item, item))
    break # 只处理当前文件夹
# zq_log.log_debug('Tools下所有模块引入完成')


name = 'zqpy'
import sys, os

dir_path = os.path.dirname(__file__).replace('\\','/')
dir_name = os.path.basename(dir_path)
for _root, _dirs, _files in os.walk(dir_path):
    sys.path.append(_root)
    imports = []
    # 文件形式引入
    for _file in _files:
        if not _file in ['__init__.py', 'setup.py', 'setup1.py'] and _file.endswith('.py'):
            _file = _file.replace('.py','')
            imports.append(_file)
    # 文件夹形式引入
    for _dir in _dirs:
        if not _dir in ['__pycache__', 'pipy']:
            imports.append(_dir)

    for item in imports:
        print('debug: import %s.%s'%(dir_name, item))
        exec('import %s'%(item))
        # exec('if hasattr(%s, "docs"): %s.docs()'%(item, item))
    break # 只处理当前文件夹
zq_log.log_debug('Modules下所有模块引入完成, 后面整理不常用的类，将import挪动到class类，赋值给self的形式调用，防止不用又在引用')
# class xx():
#     def __init__(self):
#         import time
#         self.time = time
#         print(66)
#     def aaa(self):
#         print(self.time.time())
# xx().aaa()



# https://www.runoob.com/python3/python-mongodb.html
import pymongo

class zq_mongodb_class():
    def __init__(self, host='43.128.88.114', port=27072, uri=None):
        '''
        初始化：下面比选一个   不要用_id进行操作，那个是游标的，查改不到东西
        :param host:ip '43.128.88.114'
        :param port:端口 27072

        :param uri:自定义拼装地址
        '''
        if uri:
            self.myclient = pymongo.MongoClient(uri)
        else:
            self.myclient = pymongo.MongoClient(host=host, port=port)

    def mongodb_has_db(self, db_name):
        '''
        是否存在数据库
        '''
        return db_name in self.myclient.list_database_names()

    def mongodb_has_table(self, db_name, table_name):
        '''
        是否存在表
        '''
        return table_name in self.mongodb_get_db(db_name).list_collection_names()

    def mongodb_get_db(self, db_name):
        '''
        获取db数据库
        '''
        return self.myclient[db_name]

    def mongodb_get_table(self, db_name, table_name):
        '''
        获取 db 里的 表
        '''
        return self.mongodb_get_db(db_name)[table_name]
    
    def mongodb_insert(self, db_name, table_name, data:dict):
        '''
        插入单条记录
        :param data: {'_id':固定id[不填默认最后],'字段名1':'值1', '字段名2':'值3'}
        :return 类型: inserted_id插入id
        '''
        return self.mongodb_get_table(db_name, table_name).insert_one(data)
    
    def mongodb_inserts(self, db_name, table_name, data:dict):
        '''
        插入多条记录
        :param data: [{'_id':固定id[不填默认最后], '字段名1':'值1', '字段名2':'值3'},{'_id':固定id[不填默认最后], '字段名1':'值1', '字段名2':'值3'}]
        :return 类型: inserted_ids插入id列表
        '''
        self.mongodb_get_table(db_name, table_name).insert_many(data)
    
    def mongodb_find_one(self, db_name, table_name, data:dict, result_data:dict=None):
        '''
        查询一条数据
        '''
        return self.mongodb_get_table(db_name, table_name).find_one(data, projection=(result_data if result_data else {}))

    def mongodb_find(self, db_name, table_name, data:dict, result_data:dict=None, limit:int=0):
        '''
        查询自定义数据, 返回游标
        :param data: 
        1. None 所有数据
        2. {'name': 'RUNOOB' } 根据指定条件查询
        3. {'name': {'$gt': 'H'}} 高级搜索，读取name字段中第一个字母ASCII值大于 'H' 的数据，大于的修饰符条件为 {'$gt': 'H'}
        4. {'name': {'$regex': '^R'}} 使用正则表达式搜索，用于读取 name 字段中第一个字母为 'R' 的数据，正则表达式修饰符条件为 {'$regex': '^R'}  正则刘开头的内容{'$in': ['/^liu/']}
        5. { $or: [ { quantity: { $lt: 20 } }, { price: 10 } ] }  或者
        :param result_data: 
        1. None 所有数据
        2. {'字段': 值, '字段': 值}  查询所有数据,返回指定字段 要返回的 1 不返回的 0 除了_id，不能在一个对象中同时指定 0 和 1
        :param limit: 返回的最大条数
        '''
        return self.mongodb_get_table(db_name, table_name).find(data if data else {}, result_data).limit(limit)

    def mongodb_find_to_list(self, db_name, table_name, data:dict, result_data:dict=None, limit:int=0):
        '''
        查询自定义数据，返回列表
        '''
        db_all = self.mongodb_find(db_name, table_name, data, result_data, limit)
        db_all_list = []
        for db_item in db_all:
            db_all_list.append(db_item)
        return db_all_list
    
    def mongodb_find_exits(self, db_name, table_name, data:dict):
        '''
        查询是否存在于数据库，find查到的是具体数据，用的游标
        '''
        all_data = self.mongodb_find(db_name, table_name, data, limit=1)
        for item in all_data:
            return True
        return False

    def mongodb_find_sort(self, db_name, table_name, data:dict, result_data:dict, limit:int, sort:list):
        '''
        查询数据并排序
        :param sort: 排序条件 (字段/字段列表,  1 升序  -1降序)
        '''
        return self.mongodb_find(db_name, table_name, data, result_data, limit).sort(sort[0], sort[1])

    def mongodb_update_one(self, db_name, table_name, query:dict, data:dict):
        '''
        更新一条数据
        :param query: {'name':{'$regex':'^F'}} 查询条件
        :param data: {'name':{'$regex':'^F'}} 替换为数据
        :return 类型: 描述
        '''
        return self.mongodb_get_table(db_name, table_name).update_one(query, data)

    def mongodb_update(self, db_name, table_name, query:dict, data:dict):
        '''
        更新多条记录
        :param query: {'name':{'$regex':'^F'}} 查询条件
        :param data: {'$set':{'name':{'$regex':'^F'}}} 替换为数据
        :return 类型: 描述
        '''
        return self.mongodb_get_table(db_name, table_name).update_many(query, data)

    def mongodb_delete_one(self, db_name, table_name, query:dict):
        '''
        删除一条数据
        :param query: {'name':{'$regex':'^F'}} 查询条件
        :return 类型: 描述
        '''
        return self.mongodb_get_table(db_name, table_name).delete_one(query)

    def mongodb_delete(self, db_name, table_name, query:dict={}):
        '''
        删除多条记录
        :param query: {'name':{'$regex':'^F'}} 查询条件, 空 表示删除全部
        :return 类型: 描述
        '''
        return self.mongodb_get_table(db_name, table_name).delete_many(query)

    def mongodb_delete_table(self, db_name, table_name):
        '''
        删除表
        '''
        return self.mongodb_get_table(db_name, table_name).drop()
    
    def mongodb_table_num(self, db_name, table_name):
        '''
        获取表数据条数
        '''
        return self.mongodb_find(db_name, table_name, None, limit=1).count()

    def mongodb_update_or_insert(self, db_name, table_name, find_data, update_data, update_symbol='$set'):
        '''
        更新或者插入数据
        '''
        if self.mongodb_find_exits(db_name, table_name, find_data):
            self.mongodb_update(db_name, table_name, find_data,  {update_symbol:update_data})
        else:
            self.mongodb_insert(db_name, table_name, update_data)

if __name__ == '__main__':
    zq_mongodb = zq_mongodb_class()
    x = zq_mongodb.mongodb_find_one('ET1', 'DBLoginInfo', {'PlayerId':'393538333331393938333037343138313539'}, None)
    y = zq_mongodb.mongodb_find_one('ET1', 'Chat', {})
    for x in zq_mongodb.mongodb_find('ET1', 'Chat', {}, None, limit=2):
        print(x)
    for x in zq_mongodb.mongodb_find_to_list('ET1', 'Chat', {}, None, limit=2):
        print(x)
    


import win32com.client

def tts_win32com_speek(text):
    '''
    微软声音
    '''
    speak = win32com.client.Dispatch('SAPI.SpVoice')  #连接到SAPI.SpVoice COM服务
    speak.Speak(text)

def tts_win32com_req(save, text):
    '''
    微软声音保存
    '''
    fileStream = win32com.client.Dispatch("SAPI.SpFileStream")
    fileStream.Open(save, 3, False)
    speak = win32com.client.Dispatch("SAPI.SpVoice")
    speak.AudioOutputStream = fileStream
    speak.Volume = 80     # 设置音量 ，值范围是0到100，分别代表最小和最大音量级别
    speak.Rate = 2        # 设置速度 ，值介于-10到10之间，分别代表最慢的说话速度和最快的说话速度
    print(speak.Voice.GetDescription())      # 看这个语音的具体描述
    speak.Speak(text)
    fileStream.Close()
    return True





import os, sys, re
import json
import time
import psutil
import threading
import subprocess
import zq_const, zq_log, zq_http, zq_cv, zq_file

dir_path = os.path.dirname(__file__).replace('\\','/')
dir_name = os.path.basename(dir_path)
for _root, _dirs, _files in os.walk(dir_path):
    sys.path.append(_root)
    imports = []
    # 文件形式引入
    for _file in _files:
        if not _file == '__init__.py' and _file.endswith('.py'):
            _file = _file.replace('.py','')
            imports.append(_file)
    # 文件夹形式引入
    for _dir in _dirs:
        if not _dir in ['__pycache__']:
            imports.append(_dir)

    for item in imports:
        zq_log.log_debug('import zqpy.%s.%s'%(dir_name, item))
        exec('import %s'%(item))
        # exec('if hasattr(%s, "docs"): %s.docs()'%(item, item))
    break # 只处理当前文件夹
zq_log.log_debug('Tools下所有模块引入完成')


def tools_run_with_reloader(func):
    ''' 利用的flask的重启模块, 让代码在运行时，修改重启 '''
    from werkzeug._reloader import run_with_reloader
    run_with_reloader(func)

def tools_open_dir(*dirs):
    ''' 打开文件夹列表 '''
    for dir in dirs:
        if not os.path.exists(dir) and not 'storage/emulated/' in dir:
            continue
        try:
            os.startfile(dir)
        except BaseException as e:
            zq_log.log_debug('打开: %s  失败：%s'%(dir,str(e)))

def tools_kill_progress(name, key_words_path):
    '''
    杀掉进程
    name: 进程名字
    key_words_path: 进程关键字路径
    '''
    try:
        pids = psutil.pids()
        for pid in pids:
            p = psutil.Process(pid)
            if p.name() == name and key_words_path in p.exe():
                os.system('taskkill /F /IM %s'%name)
                zq_log.log_debug('杀进程: %s'%name)
    except Exception as e:
        zq_log.log_debug('杀进程: %s %s  失败：%s'%(name , key_words_path, str(e)))

def tools_async_fun(f):
    ''' 线程运行方法 '''
    def wrapper(*args, **kwargs):
        thr = threading.Thread(target = f, args = args, kwargs = kwargs)
        thr.start()
        return thr
    return wrapper

def tools_auto_install_import():
    ''' 自动引入安装没有的库 '''
    from importlib import import_module
    class AutoInstallClass():
        _loaded = set()
        @classmethod
        def find_spec(cls, name, path, target=None):
            if path is None and name not in cls._loaded:
                cls._loaded.add(name)
                zq_log.log_debug("Installing", name)
                try:
                    result = os.system('pip install {}'.format(name))
                    if result == 0:
                        return import_module(name)
                except Exception as e:
                    zq_log.log_debug("Failed", e)
                    return None
    zq_log.log_debug('引入自动导入库成功')
    sys.meta_path.append(AutoInstallClass)

#region AES加密
def tools_encrypt(data:bytes, pwd:bytes):
    ''' aes 加密bytes '''
    dataLen=len(data)
    pwdLen=len(pwd)
    #获取Key
    key=dataLen//pwdLen*pwd+pwd[:dataLen%pwdLen]
    dataList = []
    #进行循环加密
    for i in range(len(key)):
        newByte=key[i]^data[i]
        #写出二进制文件
        dataList.append(newByte)
    data = bytes(dataList)
    return data

def tools_encryptStr(content:str, pwd:str, encode:str='utf8'):
    ''' aes 加密string '''
    data = tools_encrypt(bytes(content,encode), bytes(pwd,encode))
    return str(data, encode)

def tools_encryptFile(sourcePath:str, savePath:str, pwd:str, encode:str='utf8'):
    ''' aes 加密文件 '''
    fs=open(sourcePath,mode='rb')
    data=fs.read()
    fs.close()
    data = tools_encrypt(data, bytes(pwd,encode))
    if savePath:
        fs=open(savePath,mode='wb')
        fs.write(data)
        fs.close()
    return data
#endregion AES加密

#region AES解密
def tools_decrypt(data:bytes, pwd:bytes):
    ''' aes 解密bytes '''
    return tools_encrypt(data, pwd)
    
def tools_decryptStr(content:str, pwd:str, encode:str='utf8'):
    ''' aes 解密string '''
    return tools_encryptStr(content, pwd, encode)
    
def tools_decryptFile(sourcePath:str, savePath:str, pwd:str, encode:str='utf8'):
    ''' aes 解密文件 '''
    return tools_encryptFile(sourcePath, savePath, pwd, encode)
#endregion AES解密

#region 正则相关
def tools_regex_url(content):
    ''' 获取URL  返回数组 '''
    return re.findall('https?://(?:[-\w.]|(?:%[\da-fA-F]{2})*/)+', content)

def tools_regex_double_quotation(content):
    ''' 获取"" 中的内容  返回数组 '''
    return re.findall('"(.*)"', content)

def tools_regex_between_content(a, b, content):
    ''' 获取a b 之间的内容  返回数组 '''
    return re.findall('.*%s(.*)%s.*'%(a,b),content)

def tools_regex_ip(content):
    ''' 正则获取IP地址 '''
    return re.findall(r'(?<![\.\d])(?:\d{1,3}\.){3}\d{1,3}(?![\.\d])', content)

def tools_regex_text(text, lenth):
    ''' 正则截断字符串【按照指定长度】 '''
    textArr = re.findall('.{'+str(lenth)+'}', text)
    textArr.append(text[(len(textArr)*lenth):])
    return textArr

def tools_regex_number(content, start=''):
    '''正则获取数字start后面的数字'''
    return re.findall('%s%s%s'%(r'(?<=',start,r')\d+\.?\d*'), content)

def tools_regex_all_number(content):
    ''' 正则所有的数字 '''
    return re.findall(r'\d+', content)

#endregion

#region zip相关
import zlib, zipfile
# http://api.1473.cn/LearningMaterials/Node/Node_zlib.aspx
# https://www.cnblogs.com/ManyQian/p/9193199.html

# zlib.compress 用来压缩字符串的bytes类型
def tools_zip_str_compress(message):
    '''字符串加密'''
    bytes_message = message.encode()
    return zlib.compress(bytes_message, zlib.Z_BEST_COMPRESSION)
    
def tools_zip_str_decompress(compressed):
    '''字符串解密'''
    return zlib.decompress(compressed)

# zlib.compressobj 用来压缩数据流，用于文件传输 level[1-9]
def tools_zip_file_compress(inputFile, zlibFile, level=9):
    '''文件加密'''
    infile = open(inputFile, 'rb')
    zfile = open(zlibFile, 'wb')
    compressobj = zlib.compressobj(level)   # 压缩对象
    data = infile.read(1024)                # 1024为读取的size参数
    while data:
        zfile.write(compressobj.compress(data))     # 写入压缩数据
        data = infile.read(1024)        # 继续读取文件中的下一个size的内容
    zfile.write(compressobj.flush())    # compressobj.flush()包含剩余压缩输出的字节对象，将剩余的字节内容写入到目标文件中

def tools_zip_file_decompress(zlibFile, endFile):
    '''文件解密'''
    zlibFile = open(zlibFile, 'rb')
    endFile = open(endFile, 'wb')
    decompressobj = zlib.decompressobj()
    data = zlibFile.read(1024)
    while data:
        endFile.write(decompressobj.decompress(data))
        data = zlibFile.read(1024)
    endFile.write(decompressobj.flush())

def tools_zip_collect_file(input_path, result):
    '''
    对目录进行深度优先遍历
    :param input_path:
    :param result:
    :return:
    '''
    files = os.listdir(input_path)
    for file in files:
        if os.path.isdir(input_path + '/' + file):
            tools_zip_collect_file(input_path + '/' + file, result)
        else:
            result.append(input_path + '/' + file)

def tools_zip_dir(input_path, output_path, output_name, password=None):
    '''
    压缩文件夹
    :param input_path: 要被压缩的文件夹路径
    :param output_path: 打出得解压包的路径
    :param output_name: 压缩包名称
    :param password: 密码
    :return:
    '''
    zq_log.log_debug('==开始压缩==')
    output_path = zq_file.dir_auto_path_merage(output_path, output_name)
    zipfiles = zipfile.ZipFile(output_path, 'w', zipfile.ZIP_DEFLATED)
    filelists = []
    tools_zip_collect_file(input_path, filelists)
    for file in filelists:
        if file != output_path:
            zipfiles.write(file, file.replace(input_path, ''))
    zipfiles.setpassword((password if password else '666666').encode())
    # 调用了close方法才会保证完成压缩
    zipfiles.close()
    zq_log.log_debug('==压缩成功==')
    return output_path

def tools_zip_unzip_dir(output_path, input_path, password):
    '''
    解压缩文件夹
    :param output_path: 解压缩包的路径
    :param input_path: 压缩文件的路径
    :param password: 密码
    :return:
    '''
    zq_log.log_debug('==开始解压==')
    isok = zipfile.is_zipfile(input_path)
    if isok:
        zipfiles=zipfile.ZipFile(input_path,'r')
        zq_log.log_debug(output_path if output_path else os.path.splitext(input_path)[0])
        zipfiles.extractall(os.path.join(output_path if output_path else os.path.splitext(input_path)[0]),pwd=(password if password else '666666').encode())
        zipfiles.close()
        zq_log.log_debug('==解压成功==')
    else:
        zq_log.log_debug('这不是一个正规的zip压缩包')

def tools_zip_file_by_volume(file_path, block_size, del_source_file=False):
    ''''''
    '''
    文件分卷压缩
    :param file_path: 文件路径
    :param block_size: 分卷大小(b) 1M=1*1024*1024
    :param del_source_file: 是否删除源文件
    :return 压缩有路径
    '''
    file_size = zq_file.file_size(file_path)  # 文件字节数
    path, file_name, suffix = zq_file.file_split(file_path)  # 除去文件名以外的path，文件名, 文件后缀名
    # 添加到临时压缩文件
    zip_file = file_path + '.zip'
    with zipfile.ZipFile(zip_file, 'w') as zf:
        zf.write(file_path, arcname=file_name)
    # 小于分卷尺寸则直接返回压缩文件路径
    if file_size > block_size:
        fp = open(zip_file, 'rb')
        count = file_size // block_size + 1
        # 创建分卷压缩文件的保存路径
        save_dir = path + os.sep + file_name + '_split'
        if os.path.exists(save_dir):
            from shutil import rmtree
            rmtree(save_dir)
        os.mkdir(save_dir)
        # 拆分压缩包为分卷文件
        for i in range(1, count + 1):
            _suffix = 'z{:0>2}'.format(i) if i != count else 'zip'
            name = save_dir + os.sep + file_name.replace(str(suffix), _suffix)
            f = open(name, 'wb+')
            f.write(fp.read(block_size))
        fp.close()
        os.remove(zip_file)     # 删除临时的 zip 文件
        zip_file = save_dir
    if del_source_file:
        os.remove(file_path)
    return zip_file


def tools_zip_unzip_file_by_volume(dir_path, del_source_dir=False):
    '''
    合并多个分卷文件
    :param dir_path: 文件路径
    :return 解压后的文件路径
    '''
    if not '_split' in dir_path:
        dir_path = '%s%s'%(dir_path, '_split')
    zippath = dir_path.replace('_split', '')
    zippath_temp = '%s_temp'%zippath
    filenames = zq_file.dir_get_files(dir_path, True)
    filenames = zq_file.file_sort(filenames)
    with open(zippath_temp, 'wb+') as outfile:
        for fname in filenames:
            with open(fname, 'rb') as infile:
                outfile.write(infile.read())
    tools_zip_unzip_dir(os.path.dirname(zippath), zippath_temp, None)
    os.remove(zippath_temp)
    if del_source_dir:
        zq_file.dir_remove(dir_path, True)
    return zippath

#endregion
#region 字符串相关
def tools_string_to_list(strValue, splite):
    ''' 字符串转列表 去头尾空格 '''
    spliteList = strValue.split(splite)
    spliteList = [i.strip() for i in spliteList]
    return spliteList
#endregion


#region selenium 操作

def tools_track(distance):
    """
    根据计算中的滑块 偏移量获取移动轨迹
    :param distance: 偏移量
    :return: 移动轨迹
    """
    # 移动轨迹
    track = []
    # 当前位移
    current = 0
    # 减速阈值
    mid = distance * 4 / 5
    # 计算间隔
    t = 0.2
    # 初速度
    v = 100

    while current < distance:
        if current < mid:
            # 加速度为正1
            a = 10
        else:
            # 加速度为负1
            a = -15
        # 初速度v0
        v0 = v
        # 当前速度v = v0 + at
        v = v0 + a * t
        # 移动距离x = v0t + 1/2 * a * t^2
        move = v0 * t + 1 / 2 * a * t * t
        # 当前位移
        current += move
        # 加入轨迹
        track.append(round(move))
    track.append(distance - current)
    return track

def tools_move_to_gap(selenium_tools, slider, bgUrl, itemUrl, scale):
    """
    智能识别并拖动滑动模块
    :param slider: 移动的滑块 selenium
    :param bgUrl: 背景url
    :param itemUrl: 缺口url
    :param scale: 图片和网页真实渲染的比例 如 # itemPath 得到大小是110 实际渲染大小是68
    """
    bgPath = zq_const.const_merage_caches_dir('cv', 'bg.jpg')
    itemPath = zq_const.const_merage_caches_dir('cv', 'item.png')
    outPath = zq_const.const_merage_caches_dir('cv', 'out.jpg')
    zq_http.https.http_download(bgUrl, bgPath, False)
    zq_http.https.http_download(itemUrl, itemPath, False)
    offset = zq_cv.cv_img_contain_img_points_by_cv2(bgPath, itemPath, outPath)
    offset = offset*scale
    tracks = tools_track(offset)
    selenium_tools.web_node_action_chains_drag(slider, tracks)

#endregion

#region 打包相关

# tools_build('./gui/gui.py', ['./douyin_web/dy_web_selenium.py','./douyin_web/dy_video_command.py'], ['./gui/configs/'], ['./caches/configs'],'./gui/caches/build')
def tools_build(main_py, sub_pys:list, source_configs:list, target_configs:list, out_path:str='./caches/dist'):
    '''
    打包管理 Pyinstaller
    :param main_py_name: 主python入口文件名
    :param sub_py_names: 子python文件名
    :param source_configs: 来源配置
    :param target_configs: 复制到的目标配置 代码自动组合 out_path/target_configs
    :param out_path: 打包路径
    '''
    zq_log.log_debug('开始打包...')

    if out_path == './caches/dist':
        out_path = zq_const.const_merage_caches_dir(out_path)
    zq_file.dir_remove('./dist', True)
    zq_file.dir_remove(out_path, True)
    zq_log.log_debug('刪除上次打包结果完成...')
    command = 'Pyinstaller -F %s'%(main_py)
    if sub_pys and len(sub_pys) > 0:
        sub_py_p = ' -p '.join(sub_pys)
        sub_py_imports = [os.path.basename(sub_item).replace('.py', '') for sub_item in sub_pys]
        sub_py_import = ' --hidden-import '.join(sub_py_imports)
        command = '%s -p %s --hidden-import %s'%(command, sub_py_p, sub_py_import)
    zq_log.log_debug(command)
    popen = subprocess.Popen(command)
    popen.wait()
    zq_log.log_debug('打包完成...')
    zq_file.dir_copy('./dist', out_path)
    zq_file.file_copy('./%s.spec'%os.path.basename(main_py).replace('.py',''), '%s/%s.spec'%(out_path, zq_file.file_split(main_py)[1].replace('.py','')))
    zq_log.log_debug('复制exe打包结果完成...')

    zq_file.dir_remove('./build', True)
    zq_file.dir_remove('./dist', True)
    os.remove('./%s.spec'%os.path.basename(main_py).replace('.py',''))
    zq_log.log_debug('刪除多余文件完成...')

    for index in range(len(source_configs)):
        source_item = source_configs[index]
        target_item = target_configs[index]
        if os.listdir(source_item):
            zq_file.dir_copy(source_item, zq_file.dir_auto_path_merage(out_path, target_item))
        else:
            zq_file.file_copy(source_item, zq_file.dir_auto_path_merage(out_path, target_item))
    zq_log.log_debug('复制配置文件完成...')

    zip_pkg_path = zq_file.dir_auto_path_merage(out_path, '/pkg/')
    zq_file.dir_auto_create(zip_pkg_path)
    tools_zip_dir(out_path, zip_pkg_path, 'build.zip')
    zq_log.log_debug('压缩包完成...')

    zq_log.log_debug('全部完成')
#endregion


import openpyxl
import os

#region excel类形式
class excel_class():
    def CreateExcel(self, excelName, isForce=False):
        if os.path.exists(excelName):
            if isForce:
                os.remove(excelName)
            else:
                return True
        wb = openpyxl.Workbook()
        wb.save(excelName)
    def CreateTable(self, excelName, tableNameList, tableDataDic=None):
        '''tableDataDic['表名': [行(int)[列(int)],], [行(int)[列(int)],], ]'''
        wb = openpyxl.load_workbook(excelName)
        for tableName in tableNameList:
            wb.create_sheet(tableName)
            wb.save(excelName)
        for tableName in tableNameList:
            self.WriteTableData(excelName, tableName, tableDataDic and tableDataDic.get(tableName) or False)
        
    def WriteTableData(self, excelName, tableName, tableData):
        '''tableData[行(int)[列(int)], [行(int)[列(int)], ]'''
        if tableData and tableName:
            wb = openpyxl.load_workbook(excelName)
            for hItemIndex in tableData:
                for vItemIndex in tableData[hItemIndex]:
                    wb[tableName].cell(row = hItemIndex+1,column = vItemIndex+1,value = tableData[hItemIndex][vItemIndex])
            wb.save(excelName)
    def ReadTableData(self, excelName, tableName, dataIndexList=None):
        '''
        @param: dataIndexList {行[列(int)]}
        return [行(int)[列(int)]]
        '''
        if not os.path.exists(excelName):
            return None
        wb = openpyxl.load_workbook(excelName)
        sheet = wb[tableName]
        maxH = sheet.max_row      #获取最大行数
        maxV = sheet.max_column      #获取最大列数
        list = {}
        if not dataIndexList:
            for itemH in range(maxH):
                list1 = {}
                for itemV in range(maxV):
                    list1[itemV] = sheet.cell(row = itemH+1,column = itemV+1).value
                list[itemH] = list1
        else:
            for itemH in dataIndexList:
                list1 = {}
                for itemV in dataIndexList[itemH]:
                    list1[itemV] = sheet.cell(row=itemH+1, column=itemV+1).value
                list[itemH] = list1

        return list if len(list)!=0 else None
    def ReadTableData_By_ID(self, excelName, tableName, keyIndex, value, dataIndexList=None):
        dataAll = self.ReadTableData(excelName, tableName, dataIndexList)
        for item in dataAll:
            if dataAll[item][keyIndex] == value:
                return dataAll[item]
        return None
    def ChangeTableData(self, excelName, tableName, tableDataDic):
        '''tableDataDic {行{列}}'''
        if not os.path.exists(excelName):
            return None
        wb = openpyxl.load_workbook(excelName)
        sheet = wb[tableName]
        for itemH in tableDataDic:
            for itemV in tableDataDic[itemH]:
                sheet.cell(row=itemH+1, column=itemV+1, value=tableDataDic[itemH][itemV])
        wb.save(excelName)
    def GetMaxHV(self, excelName, tableName):
        '''
        @param: dataIndexList {行[列(int)]}
        return [行(int)[列(int)]]
        '''
        if not os.path.exists(excelName):
            return None
        wb = openpyxl.load_workbook(excelName)
        sheet = wb[tableName]
        maxH = sheet.max_row      #获取最大行数
        maxV = sheet.max_column      #获取最大列数
        return (maxH, maxV)
#endregion

#region excel单体形式
def excel_create(excelName, isForce=False):
    if os.path.exists(excelName):
        if isForce:
            os.remove(excelName)
        else:
            return True
    wb = openpyxl.Workbook()
    wb.save(excelName)
def excel_create_table(excelName, tableNameList, tableDataDic=None, isExitsTabDel=False):
    '''tableDataDic['表名': [行(int)[列(int)],], [行(int)[列(int)],], ]'''
    wb = openpyxl.load_workbook(excelName)
    for tableName in tableNameList:
        if tableName in wb:
            if isExitsTabDel:
                del wb[tableName]
                wb.create_sheet(tableName)
        else:
            wb.create_sheet(tableName)
        wb.save(excelName)
    for tableName in tableNameList:
        excel_write_table(excelName, tableName, tableDataDic and tableDataDic.get(tableName) or False)
def excel_write_table(excelName, tableName, tableData):
    '''tableData[行(int)[列(int)], [行(int)[列(int)], ]'''
    if tableData and tableName:
        wb = openpyxl.load_workbook(excelName)
        for hItemIndex in tableData:
            for vItemIndex in tableData[hItemIndex]:
                wb[tableName].cell(row = hItemIndex+1,column = vItemIndex+1,value = tableData[hItemIndex][vItemIndex])
        wb.save(excelName)
def excel_read_table(excelName, tableName, dataIndexList=None):
    '''
    @param: dataIndexList {行[列(int)]}
    return [行(int)[列(int)]]
    '''
    if not os.path.exists(excelName):
        return None
    wb = openpyxl.load_workbook(excelName)
    sheet = wb[tableName]
    maxH = sheet.max_row      #获取最大行数
    maxV = sheet.max_column      #获取最大列数
    list = {}
    if not dataIndexList:
        for itemH in range(maxH):
            list1 = {}
            for itemV in range(maxV):
                list1[itemV] = sheet.cell(row = itemH+1,column = itemV+1).value
            list[itemH] = list1
    else:
        for itemH in dataIndexList:
            list1 = {}
            for itemV in dataIndexList[itemH]:
                list1[itemV] = sheet.cell(row=itemH+1, column=itemV+1).value
            list[itemH] = list1

    return list if len(list)!=0 else None
def excel_read_table_by_id(excelName, tableName, keyIndex, value, dataIndexList=None):
    dataAll = excel_read_table(excelName, tableName, dataIndexList)
    for item in dataAll:
        if dataAll[item][keyIndex] == value:
            return dataAll[item]
    return None
def excel_change_table(excelName, tableName, tableDataDic):
    '''tableDataDic {行{列}}'''
    if not os.path.exists(excelName):
        return None
    wb = openpyxl.load_workbook(excelName)
    sheet = wb[tableName]
    for itemH in tableDataDic:
        for itemV in tableDataDic[itemH]:
            sheet.cell(row=itemH+1, column=itemV+1, value=tableDataDic[itemH][itemV])
    wb.save(excelName)
def excel_get_max_hv(excelName, tableName):
    '''
    @param: dataIndexList {行[列(int)]}
    return [行(int)[列(int)]]
    '''
    if not os.path.exists(excelName):
        return None
    wb = openpyxl.load_workbook(excelName)
    sheet = wb[tableName]
    maxH = sheet.max_row      #获取最大行数
    maxV = sheet.max_column      #获取最大列数
    return (maxH, maxV)
#endregion

if __name__ == "__main__":
    excelService = excel_class()
    excelService.CreateExcel('./Test.xlsx', True)
    excelService.CreateTable('./Test.xlsx', ['Sheel1','TestT1'], {
        'Sheel1':{
            0:{0:5,1:'xx',2:'678'},
        },
        'TestT1':{
            0:{0:343,1:'xx',2:'678'},
            1:{0:343,1:'TestT1xx',2:'678TestT1'},
        }
    })
    print(excelService.ReadTableData('./Test.xlsx', 'TestT1'))
    print(excelService.ReadTableData('./Test.xlsx', 'TestT1', {0:[0,2],1:[1,2]}))
    excelService.ChangeTableData('./Test.xlsx', 'TestT1',{0:{0:'996',1:'一个大西瓜'}})


import time, os, subprocess, re
import zq_log

def adb_execute(adbCommand, device = None, shell = True, stdout = subprocess.PIPE):
    ''' adb 方法执行 '''
    if device:
        device = device.replace('_a_','')
        device = device.replace('_b_','-')
        device = device.replace('_c_','.')
        device = device.replace('_d_',':')
        adbCommand = adbCommand.replace('adb ', 'adb -s %s '%(device))
    zq_log.log_debug(adbCommand)
    popen = subprocess.Popen(adbCommand,shell=shell,stdout=stdout)
    popen.wait()
    if stdout == subprocess.PIPE:
        readLine = popen.stdout.readlines()
        return readLine
    else:
        return True

def adb_get_copy_value(device = None):
    ''' 获取复制的值 '''
    adb_execute('adb shell am startservice ca.zgrs.clipper/.ClipboardService', device=device)
    readLine = adb_execute('adb shell am broadcast -a clipper.get', device=device)
    if len(readLine) < 2:
        return ''
    resultData = str(readLine[1], 'utf8')
    return resultData

def adb_kill_app(packageName, device = None):
    ''' 杀掉某APP '''
    return adb_execute('adb shell "am force-stop %s"'%(packageName), device=device)     #退出后台但是不会清理数据
    # zq_log.log_debug('adb shell pm clear %s'%(packageName.split('/')[0]))
    # return adb_execute('adb shell pm clear %s'%(packageName.split('/')[0]), device=device)

def adb_show_app(appPackage, appLayer, device = None):
    ''' 显示某个应用的 某层 '''
    return adb_execute('adb shell am start -n %s/%s'%(appPackage,appLayer), device=device)

def adb_get_app_curr(device = None):
    ''' 查看当前所在的页面 属于的package 和 活动层，方便打开用 '''
    readLine = adb_execute('adb shell dumpsys window windows | findstr "Current"', device=device,shell=True,stdout=subprocess.PIPE)
    return readLine

def adb_click(x, y, device = None):
    ''' ADB点击 '''
    return adb_execute('adb shell input tap %s %s'%(x,y), device=device)

def adb_drag_and_drop(x1, y1, x2, y2, device = None):
    ''' ADB按住滑动 '''
    return adb_execute('adb shell input draganddrop %s %s %s %s'%(x1, y1, x2, y2), device=device)

def adb_swipe(x1, y1, x2, y2, device = None):
    ''' ADB滑动 '''
    return adb_execute('adb shell input swipe %s %s %s %s'%(x1, y1, x2, y2), device=device)

def adb_input_key_event(content = '', device = None):
    ''' 输入键 '''
    return adb_execute('adb shell input keyevent "%s"'%(content), device=device)

def adb_input_text(content = '', device = None):
    ''' 输入内容，不支持中文  模拟器以 /mnt开头 + /sdcard手机最高目录名字 '''
    return adb_execute('adb shell input text "%s"'%(content), device=device)
    # return adb_execute('adb shell am broadcast -a ADB_INPUT_TEXT --es msg "%s"'%(content))

def adb_input_text_by_app(content = '', device = None):
    ''' 输入内容，支持中文  必须满足 1.安装了ADBKeyboard.apk  2.adb 1.0.32以上 '''
    return adb_execute('adb shell am broadcast -a ADB_INPUT_TEXT --es msg "%s"'%(content), device=device)

def adb_push_file(pcPath, androidPath, device = None):
    ''' 输电脑文件复制到手机上，手机测不支持中文 '''
    return adb_execute('adb push %s %s'%(pcPath, androidPath), device=device,stdout = subprocess.DEVNULL) # 小文件可以用subprocess.PIPE，大文件用它要崩，一直卡

def adb_pull_file(androidPath, pcPath, device = None):
    ''' 手机文件复制到电脑上，手机测不支持中文 '''
    return adb_execute('adb pull %s %s'%(androidPath, pcPath), device=device)

def adb_screenshot(targetPath='', device = None):
    '''
    手机截图
    :param targetPath: 目标路径
    :return:
    '''
    format_time = int(time.time())
    adb_execute('adb shell screencap -p /sdcard/%s.png'%format_time, device=device)
    time.sleep(1)
    outPath = targetPath
    if targetPath == '':
        outPath = os.path.dirname(__file__)
    adb_pull_file('/sdcard/%s.png' % (format_time,), outPath)
    adb_remove_file('/sdcard/%s.png' % (format_time,))
    time.sleep(1)
    return outPath if ('.jpg' in outPath or '.png' in outPath) else ('%s%s.png' % (outPath, format_time,))

def adb_remove_file(path):
    '''
    从手机端删除文件
    :return:
    '''
    adb_execute('adb shell rm %s'%path)

def adb_remove_dir(path):
    '''
    从手机端删除目录
    :return:
    '''
    adb_execute('adb shell rm -r %s'%path)

def adb_get_devices(device = None):
    ''' 获取设备信息列表 '''
    devices = []
    readLine = adb_execute('adb devices', device=device) 
    for item in readLine:
        item = item.decode('utf-8')
        items = item.split('\t')
        if len(items) == 2:
            newItems = []
            for itemTemp in items:
                itemTemp = itemTemp.strip().replace('-','_b_')
                itemTemp = itemTemp.replace('.','_c_')
                itemTemp = itemTemp.replace(':','_d_')
                if itemTemp.split('_')[0].isdigit():
                    itemTemp = '_a_' + itemTemp
                newItems.append(itemTemp)
            devices.append(newItems)
    return devices

def adb_open_uri(uri, device=None):
    ''' 打开Uri '''
    return adb_execute('adb shell am start -a android.intent.action.VIEW -d '%uri, device=device)

def adb_create_mkdir(path, device=None):
    '''
    创建目录
    :param path: 路径
    :return:
    '''
    return adb_execute('adb shell mkdir -p %s'%path)


def adb_dump_apk(path):
    '''
    dump apk文件
    :param path: apk路径
    :return:
    '''
    # 检查build-tools是否添加到环境变量中
    # 需要用到里面的aapt命令
    l = os.environ['PATH'].split(';')
    build_tools = False
    for i in l:
        if 'build-tools' in i:
            build_tools = True
    if not build_tools:
        raise EnvironmentError('ANDROID_ADB BUILD-TOOLS COMMAND NOT FOUND.\nPlease set the environment variable.')
    return os.popen('aapt dump badging %s' % (path,))

def adb_dump_xml(path, filename):
    '''
    dump apk xml文件
    :return:
    '''
    return os.popen('aapt dump xmlstrings %s %s' % (path, filename))

def adb_uiautomator_dump():
    '''
    获取屏幕uiautomator xml文件
    :return:
    '''
    return adb_execute('adb shell uiautomator dump').read().split()[-1]

def adb_clear_app_data(package):
    '''
    清理应用数据
    :return:
    '''
    return adb_execute('adb shell pm clear %s' % (package,))
 
def adb_install(path, device = None):
    '''
    安装apk文件
    :return:
    '''
    # adb install 安装错误常见列表
    errors = {'INSTALL_FAILED_ALREADY_EXISTS': '程序已经存在',
                'INSTALL_DEVICES_NOT_FOUND': '找不到设备',
                'INSTALL_FAILED_DEVICE_OFFLINE': '设备离线',
                'INSTALL_FAILED_INVALID_APK': '无效的APK',
                'INSTALL_FAILED_INVALID_URI': '无效的链接',
                'INSTALL_FAILED_INSUFFICIENT_STORAGE': '没有足够的存储空间',
                'INSTALL_FAILED_DUPLICATE_PACKAGE': '已存在同名程序',
                'INSTALL_FAILED_NO_SHARED_USER': '要求的共享用户不存在',
                'INSTALL_FAILED_UPDATE_INCOMPATIBLE': '版本不能共存',
                'INSTALL_FAILED_SHARED_USER_INCOMPATIBLE': '需求的共享用户签名错误',
                'INSTALL_FAILED_MISSING_SHARED_LIBRARY': '需求的共享库已丢失',
                'INSTALL_FAILED_REPLACE_COULDNT_DELETE': '需求的共享库无效',
                'INSTALL_FAILED_DEXOPT': 'dex优化验证失败',
                'INSTALL_FAILED_DEVICE_NOSPACE': '手机存储空间不足导致apk拷贝失败',
                'INSTALL_FAILED_DEVICE_COPY_FAILED': '文件拷贝失败',
                'INSTALL_FAILED_OLDER_SDK': '系统版本过旧',
                'INSTALL_FAILED_CONFLICTING_PROVIDER': '存在同名的内容提供者',
                'INSTALL_FAILED_NEWER_SDK': '系统版本过新',
                'INSTALL_FAILED_TEST_ONLY': '调用者不被允许测试的测试程序',
                'INSTALL_FAILED_CPU_ABI_INCOMPATIBLE': '包含的本机代码不兼容',
                'CPU_ABIINSTALL_FAILED_MISSING_FEATURE': '使用了一个无效的特性',
                'INSTALL_FAILED_CONTAINER_ERROR': 'SD卡访问失败',
                'INSTALL_FAILED_INVALID_INSTALL_LOCATION': '无效的安装路径',
                'INSTALL_FAILED_MEDIA_UNAVAILABLE': 'SD卡不存在',
                'INSTALL_FAILED_INTERNAL_ERROR': '系统问题导致安装失败',
                'INSTALL_PARSE_FAILED_NO_CERTIFICATES': '文件未通过认证 >> 设置开启未知来源',
                'INSTALL_PARSE_FAILED_INCONSISTENT_CERTIFICATES': '文件认证不一致 >> 先卸载原来的再安装',
                'INSTALL_FAILED_INVALID_ZIP_FILE': '非法的zip文件 >> 先卸载原来的再安装',
                'INSTALL_CANCELED_BY_USER': '需要用户确认才可进行安装',
                'INSTALL_FAILED_VERIFICATION_FAILURE': '验证失败 >> 尝试重启手机',
                'DEFAULT': '未知错误'
                }
    zq_log.log_debug('Installing...')
    l = adb_execute('adb install -r %s' % (path,), device).read()
    if 'Success' in l:
        zq_log.log_debug('adb_install Success')
    if 'Failure' in l:
        reg = re.compile('\\[(.+?)\\]')
        key = re.findall(reg, l)[0]
        try:
            zq_log.log_debug('install Failure >> %s' % errors[key])
        except KeyError:
            zq_log.log_debug('install Failure >> %s' % key)
    return l

def adb_uninstall(package, device):
    '''
    卸载apk
    :param package: 包名
    :return:
    '''
    zq_log.log_debug('Uninstalling...')
    l = adb_execute('adb uninstall %s' % (package,), device).read()
    zq_log.log_debug(l)
 
def adb_get_cache_logcat():
    '''
    导出缓存日志
    :return:
    '''
    return adb_execute('adb logcat -v time -d')
 
def adb_get_crash_logcat():
    '''
    导出崩溃日志
    :return:
    '''
    return adb_execute('adb logcat -v time -d | %s AndroidRuntime' % ('findstr',))

def adb_clear_cache_logcat():
    '''
    清理缓存区日志
    :return:
    '''
    return adb_execute('adb logcat -c')

def adb_get_device_time():
    '''
    获取设备时间
    :return:
    '''
    return adb_execute('adb shell date').read().strip()
 
def adb_ls(command):
    '''
    shell ls命令
    :return:
    '''
    return adb_execute('adb shell ls %s' % (command,)).readlines()
 
def adb_file_exists(target):
    '''
    判断文件在目标路径是否存在
    :return:
    '''
    l = adb_ls(target)
    for i in l:
        if i.strip() == target:
            return True
    return False
 
def adb_is_install(target_app):
    '''
    判断目标app在设备上是否已安装
    :param target_app: 目标app包名
    :return: bool
    '''
    return target_app in adb_execute('adb shell pm list packages %s' % (target_app,)).read()
 
def adb_get_device_model():
    '''
    获取设备型号
    :return:
    '''
    return adb_execute('adb shell getprop ro.product.model').read().strip()
 
def adb_get_device_id():
    '''
    获取设备id
    :return:
    '''
    return adb_execute('adb get-serialno').read().strip()
 
def adb_get_device_android_version():
    '''
    获取设备Android版本
    :return:
    '''
    return adb_execute('adb shell getprop ro.build.version.release').read().strip()

def adb_get_device_sdk_version():
    '''
    获取设备SDK版本
    :return:
    '''
    return adb_execute('adb shell getprop ro.build.version.sdk').read().strip()

def adb_get_device_mac_address():
    '''
    获取设备MAC地址
    :return:
    '''
    return adb_execute('adb shell cat /sys/class/net/wlan0/address').read().strip()

def adb_get_device_ip_address():
    '''
    获取设备IP地址
    pass: 适用WIFI 蜂窝数据
    :return:
    '''
    if not adb_get_wifi_state() and not adb_get_data_state():
        return
    l = adb_execute('adb shell ip addr | %s global' % 'findstr').read()
    reg = re.compile('\d+\.\d+\.\d+\.\d+')
    return re.findall(reg, l)[0]

def adb_root():
    '''
    获取root状态
    :return:
    '''
    return 'not found' not in adb_execute('adb shell su -c ls -l /data/').read().strip()

def adb_get_device_imei():
    '''
    获取设备IMEI
    :return:
    '''
    sdk = adb_get_device_sdk_version()
    # Android 5.0以下方法
    if int(sdk) < 21:
        l = adb_execute('adb shell dumpsys iphonesubinfo').read()
        reg = re.compile('[0-9]{15}')
        return re.findall(reg, l)[0]
    elif adb_root():
        l = adb_execute('adb shell service call iphonesubinfo 1').read()
        zq_log.log_debug(l)
        zq_log.log_debug(re.findall(re.compile("'.+?'"), l))
        imei = ''
        for i in re.findall(re.compile("'.+?'"), l):
            imei += i.replace('.', '').replace("'", '').replace(' ', '')
        return imei
    else:
        zq_log.log_debug('The device not root.')
        return ''

def adb_check_sim_card():
    '''
    检查设备SIM卡
    :return:
    '''
    return len(adb_execute('adb shell getprop | %s gsm.operator.alpha]' % 'findstr').read().strip().split()[-1]) > 2

def adb_get_device_operators():
    '''
    获取运营商
    :return:
    '''
    return adb_execute('adb shell getprop | %s gsm.operator.alpha]' % 'findstr').read().strip().split()[-1]

def adb_get_device_state():
    '''
    获取设备状态
    :return:
    '''
    return adb_execute('adb get-state').read().strip()

def adb_get_display_state():
    '''
    获取屏幕状态
    :return: 亮屏/灭屏
    '''
    l = adb_execute('adb shell dumpsys power').readlines()
    for i in l:
        if 'mScreenOn=' in i:
            return i.split()[-1] == 'mScreenOn=true'
        if 'Display Power' in i:
            return 'ON' in i.split('=')[-1].upper()

def adb_get_screen_normal_size():
    '''
    获取设备屏幕分辨率 >> 标配
    :return:
    '''
    return adb_execute('adb shell wm size').read().strip().split()[-1].split('x')

def adb_get_screen_reality_size():
    '''
    获取设备屏幕分辨率 >> 实际分辨率
    :return:
    '''
    x = 0
    y = 0
    l = adb_execute('adb shell getevent -p | %s -e "0"' % 'findstr').readlines()
    for n in l:
        if len(n.split()) > 0:
            if n.split()[0] == '0035':
                x = int(n.split()[7].split(',')[0])
            elif n.split()[0] == '0036':
                y = int(n.split()[7].split(',')[0])
    return x, y

def adb_get_device_interior_sdcard():
    '''
    获取内部SD卡空间
    :return: (path,total,used,free,block)
    '''
    return adb_execute('adb shell df | %s \/mnt\/shell\/emulated' % 'findstr').read().strip().split()

def adb_get_device_external_sdcard():
    '''
    获取外部SD卡空间
    :return: (path,total,used,free,block)
    '''
    return adb_execute('adb shell df | %s \/storage' % 'findstr').read().strip().split()

def __fill_rom(path, stream, count):
    '''
    填充数据
    :param path: 填充地址
    :param stream: 填充流大小
    :param count: 填充次数
    :return:
    '''
    adb_execute('adb shell dd if=/dev/zero of=%s bs=%s count=%s' % (path, stream, count)).read().strip()

def adb_fill_interior_sdcard(filename, size):
    '''
    填充内置SD卡
    :param filename: 文件名
    :param size: 填充大小，单位byte
    :return:
    '''
    if size > 10485760:  # 10m
        __fill_rom('sdcard/%s' % filename, 10485760, size / 10485760)
    else:
        __fill_rom('sdcard/%s' % filename, size, 1)

def adb_fill_external_sdcard(filename, size):
    '''
    填充外置SD卡
    :param filename: 文件名
    :param size: 填充大小，单位byte
    :return:
    '''
    path = adb_get_device_external_sdcard()[0]
    if size > 10485760:  # 10m
        __fill_rom('%s/%s' % (path, filename), 10485760, size / 10485760)
    else:
        __fill_rom('%s/%s' % (path, filename), size, 1)

def adb_kill_process(pid):
    '''
    杀死进程
    pass: 一般需要权限不推荐使用
    :return:
    '''
    return adb_execute('adb shell kill %s' % pid).read().strip()

def adb_quit_app(package):
    '''
    退出应用
    :return:
    '''
    return adb_execute('adb shell am force-stop %s' % package).read().strip()

def adb_reboot():
    '''
    重启设备
    :return:
    '''
    adb_execute('adb reboot')

def adb_recovery():
    '''
    重启设备并进入recovery模式
    :return:
    '''
    adb_execute('adb reboot recovery')

def adb_fastboot():
    '''
    重启设备并进入fastboot模式
    :return:
    '''
    adb_execute('adb reboot bootloader')

def adb_wifi(power):
    '''
    开启/关闭wifi
    pass: 需要root权限
    :return:
    '''
    if not adb_root():
        zq_log.log_debug('The device not root.')
        return
    if power:
        adb_execute('adb shell su -c svc wifi enable').read().strip()
    else:
        adb_execute('adb shell su -c svc wifi disable').read().strip()

def adb_data(power):
    '''
    开启/关闭蜂窝数据
    pass: 需要root权限
    :return:
    '''
    if not adb_root():
        zq_log.log_debug('The device not root.')
        return
    if power:
        adb_execute('adb shell su -c svc data enable').read().strip()
    else:
        adb_execute('adb shell su -c svc data disable').read().strip()

def adb_get_wifi_state():
    '''
    获取WiFi连接状态
    :return:
    '''
    return 'enabled' in adb_execute('adb shell dumpsys wifi | %s ^Wi-Fi' % 'findstr').read().strip()

def adb_get_data_state():
    '''
    获取移动网络连接状态
    :return:
    '''
    return '2' in adb_execute('adb shell dumpsys telephony.registry | %s mDataConnectionState' % 'findstr').read().strip()

def adb_get_network_state():
    '''
    设备是否连上互联网
    :return:
    '''
    return 'unknown host' not in adb_execute('adb shell ping -w 1 www.baidu.com').read().strip()

def adb_get_wifi_password_list():
    '''
    获取WIFI密码列表
    :return:
    '''
    if not adb_root():
        zq_log.log_debug('The device not root.')
        return []
    l = re.findall(re.compile('ssid=".+?"\s{3}psk=".+?"'), adb_execute('adb shell su -c cat /data/misc/wifi/*.conf').read())
    return [re.findall(re.compile('".+?"'), i) for i in l]

def adb_call(number):
    '''
    拨打电话
    :param number:
    :return:
    '''
    adb_execute('adb shell am start -a android.intent.action.CALL -d tel:%s' % number)

def adb_open_url(url):
    '''
    打开网页
    :return:
    '''
    adb_execute('adb shell am start -a android.intent.action.VIEW -d %s' % url)

def adb_start_application(component):
    '''
    启动一个应用
    e.g: com.android.settings/com.android.settings.Settings
    '''
    adb_execute('adb shell am start -n %s' % component)

def adb_send_keyevent(keycode):
    '''
    发送一个按键事件
    https://developer.android.com/reference/android/view/KeyEvent.html
    :return:
    '''
    adb_execute('adb shell input keyevent %s' % keycode)

def adb_rotation_screen(param):
    '''
    旋转屏幕
    :param param: 0 >> 纵向，禁止自动旋转; 1 >> 自动旋转
    :return:
    '''
    adb_execute('adb shell /system/bin/content insert --uri content://settings/system --bind '
                'name:s:accelerometer_rotation --bind value:i:%s' % param)

def adb_instrument(command):
    '''
    启动instrument app
    :param command: 命令
    :return:
    '''
    return adb_execute('adb shell am instrument %s' % command).read()

def adb_export_apk(package, target_path='', timeout=5000):
    '''
    从设备导出应用
    :param timeout: 超时时间
    :param target_path: 导出后apk存储路径
    :param package: 包名
    :return:
    '''
    num = 0
    if target_path == '':
        adb_execute('adb pull /data/app/%s-1/base.apk %s' % (package, os.path.expanduser('~')))
        while 1:
            num += 1
            if num <= timeout:
                if os.path.exists(os.path.join(os.path.expanduser('~'), 'base.apk')):
                    os.rename(os.path.join(os.path.expanduser('~'), 'base.apk'),
                                os.path.join(os.path.expanduser('~'), '%s.apk' % package))

    else:
        adb_execute('adb pull /data/app/%s-1/base.apk %s' % (package, target_path))
        while 1:
            num += 1
            if num <= timeout:
                if os.path.exists(os.path.join(os.path.expanduser('~'), 'base.apk')):
                    os.rename(os.path.join(os.path.expanduser('~'), 'base.apk'),
                                os.path.join(os.path.expanduser('~'), '%s.apk' % package))


class ABD_KEY:
    KEYCODE_UNKNOWN = 0,
    KEYCODE_MENU = 1, 
    KEYCODE_SOFT_RIGHT = 2, 
    KEYCODE_HOME = 3  # Home键
    KEYCODE_BACK = 4  # 返回键
    KEYCODE_CALL = 5  # 拨号键
    KEYCODE_ENDCALL = 6  # 挂机键
    KEYCODE_0 = 7, 
    KEYCODE_1 = 8, 
    KEYCODE_2 = 9, 
    KEYCODE_3 = 10,
    KEYCODE_4 = 11,
    KEYCODE_5 = 12,
    KEYCODE_6 = 13,
    KEYCODE_7 = 14,
    KEYCODE_8 = 15,
    KEYCODE_9 = 16,
    KEYCODE_STAR = 17  # *
    KEYCODE_POUND = 18  # #
    KEYCODE_DPAD_UP = 19  # 导航键 >> 向上
    KEYCODE_DPAD_DOWN = 20  # 导航键 >> 向下
    KEYCODE_DPAD_LEFT = 21  # 导航键 >> 向左
    KEYCODE_DPAD_RIGHT = 22  # 导航键 >> 向右
    KEYCODE_DPAD_CENTER = 23  # 导航键 >> 确定键
    KEYCODE_VOLUME_UP = 24  # 音量+键
    KEYCODE_VOLUME_DOWN = 25  # 音量-键
    KEYCODE_POWER = 26  # 电源键
    KEYCODE_CAMERA = 27  # 拍照键
    KEYCODE_CLEAR = 28,
    KEYCODE_A = 29,
    KEYCODE_B = 30,
    KEYCODE_C = 31,
    KEYCODE_D = 32,
    KEYCODE_E = 33,
    KEYCODE_F = 34,
    KEYCODE_G = 35,
    KEYCODE_H = 36,
    KEYCODE_I = 37,
    KEYCODE_J = 38,
    KEYCODE_K = 39,
    KEYCODE_L = 40,
    KEYCODE_M = 41,
    KEYCODE_N = 42,
    KEYCODE_O = 43,
    KEYCODE_P = 44,
    KEYCODE_Q = 45,
    KEYCODE_R = 46,
    KEYCODE_S = 47,
    KEYCODE_T = 48,
    KEYCODE_U = 49,
    KEYCODE_V = 50,
    KEYCODE_W = 51,
    KEYCODE_X = 52,
    KEYCODE_Y = 53,
    KEYCODE_Z = 54,
    KEYCODE_COMMA = 55  # ,
    KEYCODE_PERIOD = 56  # .
    KEYCODE_ALT_LEFT = 57,
    KEYCODE_ALT_RIGHT = 58,
    KEYCODE_SHIFT_LEFT = 59,
    KEYCODE_SHIFT_RIGHT = 60,
    KEYCODE_TAB = 61  # Tab键
    KEYCODE_SPACE = 62  # 空格键
    KEYCODE_SYM = 63,
    KEYCODE_EXPLORER = 64,
    KEYCODE_ENVELOPE = 65,
    KEYCODE_ENTER = 66  # 回车键
    KEYCODE_DEL = 67  # 退格键
    KEYCODE_GRAVE = 68  # `
    KEYCODE_MINUS = 69  # -
    KEYCODE_EQUALS = 70  # =
    KEYCODE_LEFT_BRACKET = 71  # [
    KEYCODE_RIGHT_BRACKET = 72  # ]
    KEYCODE_BACKSLASH = 73  # \
    KEYCODE_SEMICOLON = 74  # ;
    KEYCODE_APOSTROPHE = 75  # '
    KEYCODE_SLASH = 76  # /
    KEYCODE_AT = 77  # @
    KEYCODE_NUM = 78,
    KEYCODE_HEADSETHOOK = 79,
    KEYCODE_FOCUS = 80  # 对焦键
    KEYCODE_PLUS = 81  # +
    KEYCODE_MENU = 82  # 菜单键
    KEYCODE_NOTIFICATION = 83  # 通知键
    KEYCODE_SEARCH = 84  # 搜索键
    KEYCODE_TAG_LAST = 85,
    KEYCODE_MEDIA_PLAY_PAUSE = 85  # 多媒体键 >> 播放 / 暂停
    KEYCODE_MEDIA_STOP = 86  # 多媒体键 >> 停止
    KEYCODE_MEDIA_NEXT = 87  # 多媒体键 >> 下一首
    KEYCODE_MEDIA_PREVIOUS = 88  # 多媒体键 >> 上一首
    KEYCODE_MEDIA_REWIND = 89  # 多媒体键 >> 快退
    KEYCODE_MEDIA_FAST_FORWARD = 90  # 多媒体键 >> 快进
    KEYCODE_MUTE = 91  # 话筒静音键
    KEYCODE_PAGE_UP = 92  # 向上翻页键
    KEYCODE_PAGE_DOWN = 93  # 向下翻页键
    KEYCODE_ESCAPE = 111  # ESC键
    KEYCODE_FORWARD_DEL = 112  # 删除键
    KEYCODE_CAPS_LOCK = 115  # 大写锁定键
    KEYCODE_SCROLL_LOCK = 116  # 滚动锁定键
    KEYCODE_BREAK = 121  # Break / Pause键
    KEYCODE_MOVE_HOME = 122  # 光标移动到开始键
    KEYCODE_MOVE_END = 123  # 光标移动到末尾键
    KEYCODE_INSERT = 124  # 插入键
    KEYCODE_MEDIA_PLAY = 126,    # 多媒体键 >> 播放
    KEYCODE_MEDIA_PAUSE = 127  # 多媒体键 >> 暂停
    KEYCODE_MEDIA_CLOSE = 128  # 多媒体键 >> 关闭
    KEYCODE_MEDIA_EJECT = 129  # 多媒体键 >> 弹出
    KEYCODE_MEDIA_RECORD = 130  # 多媒体键 >> 录音
    KEYCODE_NUM_LOCK = 143  # 小键盘锁
    KEYCODE_VOLUME_MUTE = 164  # 扬声器静音键
    KEYCODE_ZOOM_IN = 168  # 放大键
    KEYCODE_ZOOM_OUT = 169  # 缩小键



import os, zq_file

_curr_dir, _caches_dir, _configs_dir, _logs_dir, _datas_dir, _cookies_dir, _res_dir, _env_dir, _env_ins_dir, _env_pkg_dir = ('','','','','','','','','','')

def const_set_base_dir(curr_dir):
    '''
    设置修改配置文件存放位置
    '''
    curr_dir = os.path.abspath(curr_dir)
    global _curr_dir, _caches_dir, _configs_dir, _logs_dir, _datas_dir, _cookies_dir, _res_dir, _env_dir, _env_ins_dir, _env_pkg_dir
    _curr_dir = curr_dir
    _caches_dir = '%s/caches'%_curr_dir
    _logs_dir = '%s/logs'%_caches_dir
    _datas_dir = '%s/datas'%_curr_dir
    _configs_dir = '%s/configs'%_datas_dir
    _cookies_dir = '%s/cookies'%_datas_dir
    _res_dir = '%s/res'%_curr_dir
    _env_dir = '%s/env'%curr_dir
    _env_ins_dir = '%s/ins'%_env_dir
    _env_pkg_dir = '%s/pkg'%_env_dir


def const_base_dir():
    ''' 当前项目文件夹 '''
    zq_file.dir_auto_create(_curr_dir)
    return _curr_dir
def const_caches_dir():
    ''' 当前缓存文件夹 '''
    zq_file.dir_auto_create(_caches_dir)
    return _caches_dir
def const_configs_dir():
    ''' 当前配置文件夹 '''
    zq_file.dir_auto_create(_configs_dir)
    return _configs_dir
def const_logs_dir():
    ''' 当前日志文件夹 '''
    zq_file.dir_auto_create(_logs_dir)
    return _logs_dir
def const_datas_dir():
    ''' 当前数据文件夹 '''
    zq_file.dir_auto_create(_datas_dir)
    return _datas_dir
def const_cookies_dir():
    ''' 当前cookies数据文件夹 '''
    zq_file.dir_auto_create(_cookies_dir)
    return _cookies_dir
def const_res_dir():
    ''' 当前资源文件夹 '''
    zq_file.dir_auto_create(_res_dir)
    return _res_dir
def const_env_dir():
    ''' 当前环境文件夹 '''
    zq_file.dir_auto_create(_env_dir)
    return _env_dir
def const_env_ins_dir():
    ''' 当前环境运行文件夹 '''
    zq_file.dir_auto_create(_env_ins_dir)
    return _env_ins_dir
def const_env_pkg_dir():
    ''' 当前环境安装包文件夹 '''
    zq_file.dir_auto_create(_env_pkg_dir)
    return _env_pkg_dir


def const_merage_curr_dir(*path):
    ''' 合并当前文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_base_dir(), *path)
def const_merage_caches_dir(*path):
    ''' 合并缓存文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_caches_dir(), *path)
def const_merage_configs_dir(*path):
    ''' 合并日志文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_configs_dir(), *path)
def const_merage_logs_dir(*path):
    ''' 合并日志文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_logs_dir(), *path)
def const_merage_datas_dir(*path):
    ''' 合并数据文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_datas_dir(), *path)
def const_merage_cookies_dir(*path):
    ''' 合并cookies数据文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_cookies_dir(), *path)
def const_merage_res_dir(*path):
    ''' 合并资源文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_res_dir(), *path)
def const_merage_env_dir(*path):
    ''' 合并环境文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_env_dir(), *path)
def const_merage_env_ins_dir(*path):
    ''' 合并环境运行文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_env_ins_dir(), *path)
def const_merage_env_pkg_dir(*path):
    ''' 合并环境安装包文件夹路径 '''
    return zq_file.dir_auto_path_merage(const_env_pkg_dir(), *path)


const_set_base_dir(os.getcwd())



import subprocess, os
from xmltodict import parse as xmlParse
import zq_log

class SVN_FILE_STATES:
    Normal = 0  # 000000 正常在svn管理下的最新的文件
    RemoteLocked = 1  # 000001 云端锁定态
    LocalLocked = 2  # 000010 本地锁定态
    Locked = 3  # 000011 已锁定 state and Locked == True
    LocalMod = 4  # 000100 本地有修改需提交
    RemoteMod = 8  # 001000 远程有修改需要更新
    Conflicked = 12  # 001100 冲突 state and Conflicked == Conflicked
    UnVersioned = 16  # 010000 未提交到库
    Error = 32  # 100000 错误状态

def subprocess_popen(args):
    startupinfo = subprocess.STARTUPINFO()
    startupinfo.dwFlags = subprocess.CREATE_NEW_CONSOLE | subprocess.STARTF_USESHOWWINDOW
    startupinfo.wShowWindow = subprocess.SW_HIDE
    p = subprocess.Popen(
        args,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        startupinfo=startupinfo,
        shell=True,
    )
    rst, err = p.communicate()
    try:
        rst = str(rst, 'utf-8')
    except:
        rst = str(rst, 'gbk', errors="-ignore")
    try:
        err = str(err, 'utf-8')
    except:
        err = str(err, 'gbk', errors="-ignore")
    return rst, err

def svn_command_status_xml_sync(path):
    rst, err = subprocess_popen("svn status " + path + " -u --xml")
    if err:
        return None, err
    data = rst
    data = xmlParse(data)
    return data, None

def svn_command_get_all_file_status(rootPath):
    data, info = svn_command_status_xml_sync(rootPath)
    returnDict = {}
    lockRole = ""
    state = SVN_FILE_STATES.Normal
    if info:
        if data is None:
            state = state | SVN_FILE_STATES.UnVersioned
        else:
            state = state | SVN_FILE_STATES.Error
        return returnDict
    target = data["status"]["target"]

    if target and "entry" in target:
        iterList = []
        if not isinstance(target["entry"], list):
            iterList.append(target["entry"])
        else:
            iterList = target["entry"]
        for fileStatusItem in iterList:
            state = SVN_FILE_STATES.Normal
            filePath = fileStatusItem["@path"]
            wc_status = fileStatusItem["wc-status"]
            if "unversioned" == wc_status["@item"]:
                state = state | SVN_FILE_STATES.UnVersioned
            elif "modified" == wc_status["@item"]:
                state = state | SVN_FILE_STATES.LocalMod
            elif "repos-status" in fileStatusItem:
                repos_status = fileStatusItem["repos-status"]
                if "lock" in repos_status and "lock" not in wc_status:
                    info = repos_status["lock"]["owner"]
                    lockRole = info
                    state = state | SVN_FILE_STATES.RemoteLocked
                elif "lock" in wc_status:
                    info = wc_status["lock"]["owner"]
                    lockRole = info
                    state = state | SVN_FILE_STATES.LocalLocked
                elif "modified" == repos_status["@item"]:
                    state = state | SVN_FILE_STATES.RemoteMod
                    info = "%s is modified on remote, you need update first" % filePath
                if "modified" == wc_status["@item"]:
                    state = state | SVN_FILE_STATES.LocalMod
                    info = "%s is modified on local, you need commit first" % filePath
            returnDict[os.path.normcase(filePath)] = [state, info, lockRole]
    return returnDict

def svn_gui_command(command:str, path:list, extCommand:str='', is_wait=True):
    '''
    command: "add"; "commit"; "update"; "revert";log
    '''
    path = [i.replace('\\','/') for i in path]
    all_path = '*'.join(path)
    command_string = "cmd /c tortoiseproc.exe /command:%s /path:\"%s\" %s /closeonend 2"%(command, all_path, extCommand)
    zq_log.log_debug('svn: %s'%command_string)
    if is_wait:
        subprocess.Popen(command_string, stdin=subprocess.PIPE).wait()
    else:
        os.system(command_string)



# 监听文件变化
'''
import time
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

watch_patterns = "*.py;*.txt"   # 监控文件的模式, 所有文件，那么设置成"*" 即可
ignore_patterns = ""            # 设置忽略的文件模式
ignore_directories = False      # 是否忽略文件夹变化
case_sensitive = True           # 是否对大小写敏感
event_handler = PatternMatchingEventHandler(watch_patterns, ignore_patterns, ignore_directories, case_sensitive)

def on_created(event):
    print(f"{event.src_path}被创建")

def on_deleted(event):
    print(f"{event.src_path}被删除")

def on_modified(event):
    print(f"{event.src_path} 被修改")

def on_moved(event):
    print(f"{event.src_path}被移动到{event.dest_path}")

event_handler.on_created = on_created
event_handler.on_deleted = on_deleted
event_handler.on_modified = on_modified
event_handler.on_moved = on_moved




watch_path = "../caches/"    # 监控目录
go_recursively = True                   # 是否监控子文件夹
my_observer = Observer()
my_observer.schedule(event_handler, watch_path, recursive=go_recursively)


my_observer.start()
try:
    while True:
        time.sleep(1)
except KeyboardInterrupt:
    my_observer.stop()
    my_observer.join()
'''


import time, os, zipfile, json, requests
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver import ActionChains


from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys

import zq_cv, zq_http, zq_const

class selenium_tools_class():
    def __init__(self, **kw):
        self.By = By
        self.Keys = Keys

        chrome_path = kw.get('chrome_path', zq_const.const_merage_env_ins_dir('Google/Chrome/Application/chrome.exe')) # 浏览器路径
        chrome_driver_path = kw.get('chrome_driver_path', zq_const.const_merage_env_ins_dir('chromedriver_win_91.exe')) #浏览器调试路径
        chrome_zip_path = kw.get('chrome_zip_path', zq_const.const_merage_env_pkg_dir('Google.zip')) #浏览器包装地址
        chrome_hide_run = kw.get('chrome_hide_run', False) # 是否隐藏运行
        chrome_wait_time = kw.get('chrome_wait_time', 10) # 等待时间
        chrome_web = kw.get('chrome_web', False) #同步浏览处理
        chrome_web_wait = kw.get('chrome_web_wait', False) #等待浏览处理
        chrome_cookies = kw.get('chrome_cookies', None) #cookies
        chrome_url = kw.get('chrome_url', None) #默认打开地址
        chrome_open_new = kw.get('chrome_open_new', False) #默认打开地址

        self.web = chrome_web
        if not self.web:
            options = webdriver.ChromeOptions()
            # 详细参数：https://peter.sh/experiments/chromium-command-line-switches/
            options.add_argument('--log-level=3')
            options.add_argument('--no-sandbox') # 以最高权限运行
            options.add_argument('--use-system-clipboard') # 允许操作剪切板
            options.add_argument('--allow-silent-push') #允许不显示通知的 Web 推送通知
            # options.add_argument(r'user-data-dir=C:\Users\Administrator\AppData\Local\Google\Chrome\User Data_Backup') #将登录态带入
            options.add_experimental_option("excludeSwitches", ["enable-automation"])
            options.add_experimental_option('useAutomationExtension', False)
            options.add_argument('--disable-blink-features=AutomationControlled')

            options.add_argument("--enable-javascript")
            if chrome_hide_run:
                options.add_argument('--headless') ## 浏览器不提供可视化页面
                # options.headless = True
            options.add_experimental_option('excludeSwitches',['enable-automation'])
            # 禁用浏览器弹窗
            prefs = {
                'profile.default_content_setting_values' : {
                    'notifications' : 2  
                }
            }
            options.add_experimental_option('prefs',prefs)
            options.binary_location = chrome_path

            # 准备环境,解压浏览器包装
            if not os.path.exists(chrome_path):
                r = zipfile.is_zipfile(chrome_zip_path)
                if r:     
                    fz = zipfile.ZipFile(chrome_zip_path, 'r')
                    for file in fz.namelist():
                        fz.extract(file, './env')
                else:
                    print('这不是一个正规的zip压缩包')
            self.web = webdriver.Chrome(options=options, executable_path=chrome_driver_path, chrome_options=options)
        self.web.maximize_window()
        self.web.set_page_load_timeout(chrome_wait_time)
        self.web.set_script_timeout(chrome_wait_time)
        self.web_wait = chrome_web_wait
        self.curr_window_handle = None
        if not self.web_wait:
            self.web_wait = WebDriverWait(self.web, chrome_wait_time)
        if chrome_cookies:
            self.web_add_cookies(chrome_cookies)
        if chrome_url:
            self.curr_window_handle = self.web_open_url(chrome_url, self.curr_window_handle, open_new=chrome_open_new)
    
    def web_add_cookies(self, cookies):
        ''' 添加cookies '''
        try:
            if cookies:
                for i in cookies:
                    if 'expiry' in i:
                        i.pop('expiry')
                    self.web.add_cookie(i)
        except:
            pass
    
    def web_switch_to_window(self, window_name):
        if window_name and window_name in self.web.window_handles:
            self.web.switch_to.window(window_name)

    def web_open_url(self, url, window_handle=None, open_new=False):
        ''' 打开地址 '''
        if open_new: # 新开窗口
            js='window.open("%s");'%url
            self.web.execute_script(js)
            window_handle = self.web.window_handles[-1]
            self.web_switch_to_window(window_handle)
        else: # 不新开,永远用第一个, 或者传进来的哪一个
            window_handle = window_handle if window_handle else self.web.window_handles[0]
            self.web_switch_to_window(window_handle)
            self.web.get(url)
        return self.web.current_window_handle
    
    def web_close(self, window_handle=None):
        if window_handle:
            self.web_switch_to_window(window_handle)
        self.web.close()

    def web_refresh(self):
        ''' 刷新网页 '''
        self.web.refresh()

    def web_refresh_window(self, window_handle):
        ''' 刷新网页 '''
        self.web_switch_to_window(window_handle)
        self.web_refresh()

    def web_get_cookies(self):
        ''' 获取cookies '''
        return self.web.get_cookies()

    def web_switch_to_frame(self, name):
        ''' 主窗切换到frame '''
        self.web.switch_to.frame(name)

    def web_switch_to_default_content(self):
        ''' frame切换到主窗 '''
        self.web.switch_to.default_content()

    def web_get_requests_get(self, url):
        ''' 获取带浏览器cookies的requests模拟 '''
        requestCookies = {}
        cookies = self.web_get_cookies()
        for i in cookies:
            if 'expiry' in i:
                i.pop('expiry')
            requestCookies[i['name']] = i['value']
        return requests.get(url, cookies=requestCookies)


    def web_get_requests_post(self, url, data):
        ''' 获取带浏览器cookies的requests模拟 '''
        requestCookies = {}
        cookies = json.dumps(self.web_get_cookies())
        for i in cookies:
            if 'expiry' in i:
                i.pop('expiry')
            requestCookies[i['name']] = i['value']
        return requests.post(url, cookies=requestCookies, data=data)

    def web_node_is_hidden(self, id, idValue, index=0):
        ''' web 节点是否属于隐藏节点 '''
        return self.web_node_get_attribute(id, idValue, 'hidden', index) == 'true'

    def web_wait_node_is_hidden(self, id, idValue):
        ''' web 节点是否属于隐藏节点 '''
        return self.web_wait_node_get_attribute(id, idValue, 'hidden') == 'true'

    def web_node_get_attribute(self, id, idValue, attrName='textContent', index=0):
        ''' web 获取属性 '''
        element = self.web_node_find_element(id, idValue, index)
        if element:
            return element.get_attribute(attrName)
        else:
            return None

    def web_wait_node_get_attribute(self, id, idValue, attrName='textContent'):
        ''' web 等待出现 获取属性 '''
        element = self.web_wait_node_element(id, idValue)
        if element:
            return element.get_attribute(attrName)
        else:
            return None

    def web_node_find_elements(self, id, idValue):
        ''' web 多个获取 '''
        return self.web.find_elements(id, idValue)

    def web_node_find_element(self, id, idValue, index=0):
        ''' web 直接获取 '''
        elements = self.web_node_find_elements(id, idValue)
        if len(elements) <= index:
            return None
        return elements[index]

    def web_node_element_send_key(self, id, idValue, content, index=0):
        ''' web 直接获取 '''
        try:
            element = self.web_node_find_element(id, idValue, index)
            if not element:
                return False
            element.send_keys(content)
            return True
        except:
            return False

    def web_node_element_has(self, id, idValue):
        ''' web 是否存在 节点 '''
        elements = self.web_node_find_elements(id, idValue)
        return not len(elements) == 0
    
    
    def web_node_element_has_by_element(self, parent_element, id, idValue):
        ''' web 是否存在 节点 '''
        elements = parent_element.find_elements(id, idValue)
        return not len(elements) == 0

    def web_node_find_elements_click(self, id, idValue, indexs=0, endSleep=1, startSleep=0):
        ''' web 多个直接点击 '''
        try:
            time.sleep(startSleep)
            elements = self.web_node_find_elements( id, idValue)
            if len(elements) == 0:
                return False
            if isinstance(indexs, list) or isinstance(indexs, tuple):
                for index in indexs:
                    if index < len(elements):
                        elements[index].click()
                        time.sleep(endSleep)
            else:
                if indexs < len(elements):
                    elements[indexs].click()
                    time.sleep(endSleep)
            return True
        except:
            return False


    def web_wait_node_element(self, id, idValue):
        ''' web 等待出现 '''
        try:
            return self.web_wait.until(EC.presence_of_element_located((id, idValue)))
        except:
            return None

    def web_wait_node_element_click(self, id, idValue):
        ''' web 等待出现 点击 '''
        try:
            element = self.web_wait_node_element(id, idValue)
            if not element:
                return False
            element.click()
            return True
        except:
            return False

    def web_wait_node_element_send_keys(self, id, idValue, *content):
        ''' web 等待出现 发送值 '''
        try:
            element = self.web_wait_node_element(id, idValue)
            if not element:
                return False
            element.send_keys(content)
            return True
        except:
            return False

    def web_node_action_chains_drag(self, slider, tracks):
        ActionChains(self.web).click_and_hold(slider).perform()
        for x in tracks:
            ActionChains(self.web).move_by_offset(xoffset=x,yoffset=0).perform()
        # ActionChains(self.web).move_by_offset(xoffset=offset,yoffset=0).perform()
        ActionChains(self.web).release().perform()

    def web_node_action_chains_click_and_hold(self, element, sleep):
        '''鼠标长按下'''
        ActionChains(self.web).click_and_hold(element).perform()
        time.sleep(sleep)
        ActionChains(self.web).release().perform()

    def web_node_action_chains_click(self, element):
        '''鼠标按下'''
        self.web_node_action_chains_click_and_hold(element, 0)

    def web_node_action_chains_key_down(self, key, element, sleep):
        '''
        按下输入
        '''
        ActionChains(self.web).key_down(key, element).perform()
        time.sleep(sleep)
        ActionChains(self.web).key_up(key, element).perform()
    
    def web_node_action_chains_move_to(self, element):
        ''' 隐藏按钮需要移动上去才能操作，否则 element not interactable '''
        ActionChains(self.web).move_to_element(element).perform()



import time
from datetime import datetime
from threading import Timer
import zq_log

def time_format(mFormat, mTime = None):
    ''' 时间格式化 字符串 '''
    if mTime == None:
        mTime = time.localtime()
    return time.strftime(mFormat, mTime)

def time_file_time_str(mTime = None):
    ''' 年_月_日_时_分_秒 字符串 '''
    return time_format('%Y_%m_%d_%H_%M_%S',mTime)

def time_long_time_str(mTime = None):
    ''' 年-月-日 时:分:秒 字符串 '''
    return time_format('%Y-%m-%d %H:%M:%S',mTime)
    
def time_short_time_str(mTime = None):
    ''' 年-月-日 字符串 '''
    return time_format('%Y-%m-%d',mTime)

def time_local_time():
    ''' 当前本地时间 '''
    return time.localtime()

def time_int():
    ''' 当前时间转int '''
    return int(time.time())

def time_sleep(sleep = 0):
    ''' 时间停留 '''
    time.sleep(sleep)

def time_format_by_timestamp(timeStamp, formatStr = "%Y-%m-%d %H:%M:%S"):
    ''' 时间戳转时间字符串格式 '''
    timeArray = time.localtime(int(timeStamp))
    return time_format(formatStr, timeArray)

def time_day_zero(date):
    """根据日期获取某天凌晨时间"""
    if not date:
        return 0
    date_zero = datetime.now().replace(year=date.year, month=date.month,
                                                day=date.day, hour=0, minute=0, second=0)
    date_zero_time = int(time.mktime(date_zero.timetuple())) * 1000
    return date_zero_time

def time_curr_day_zero(isSecond):
    '''
    根据日期获取当天凌晨时间
    :param: isSecond 是否是秒
    '''
    # 今天的日期
    today_date = datetime.now().date()
    # 今天的零点
    ms = time_day_zero(today_date) 
    if isSecond:
        return int(ms/1000)
    return ms

def time_curr_time():
    ''' 当前时间 datetime '''
    return datetime.now()

def time_next_day_time(year=None, month=None, day=None, hour=0, minute=0, second=0, microsecond=0, tzinfo=None):
    ''' 获取下一天的当前时间 '''
    return time_curr_time()+datetime.timedelta(year=year, month=month, day=day, hour=hour, minute=minute, second=second, microsecond=microsecond, tzinfo=tzinfo)

def time_yesterday(timedelta=-1): 
    '''向前天'''
    yesterday = datetime.date.today() + datetime.timedelta(timedelta)
    return yesterday

def time_str_is_same_day(source_date, target_date=None):
    '''
    判断字符串是否是同一天(支持仅年月日)
    source_date:年-月-日 时:分:秒
    target_date:年-月-日 时:分:秒
    '''
    source_date = source_date.replace('/', '-')
    target_date = target_date.replace('/', '-') if target_date else None
    is_same_day = False
    try:
        date_format = '%Y-%m-%d'
        if len(source_date.split(' ')) > 1:
            date_format = '%Y-%m-%d %H:%M:%S'
        target_date = target_date if target_date else time.strftime(date_format, time.localtime(time.time()))
        source_date = datetime.strptime(source_date, date_format).date()
        target_date = datetime.strptime(target_date, date_format).date()
        is_same_day = source_date == target_date
    except Exception as e:
        zq_log.log_debug('time_str_is_same_day error: %s'%str(e))
    return is_same_day

def time_str_to_date(source_date):
    '''
    字符串时间转时间data(支持仅年月日)
    source_date:年-月-日 时:分:秒
    '''
    source_date = source_date.replace('/', '-')
    date_format = '%Y-%m-%d'
    if len(source_date.split(' ')) > 1:
        date_format = '%Y-%m-%d %H:%M:%S'
    return datetime.strptime(source_date, date_format)

def time_compare_date(source_date, target_date=None):
    '''
    判断时间字符串前一个是否比后一个大(支持仅年月日)
    source_date:年-月-日 时:分:秒
    target_date:年-月-日 时:分:秒
    '''
    source_date = source_date.replace('/', '-')
    target_date = target_date.replace('/', '-') if target_date else None
    compare = 0
    try:
        date_format = '%Y-%m-%d'
        if len(source_date.split(' ')) > 1:
            date_format = '%Y-%m-%d %H:%M:%S'
        target_date = target_date if target_date else time.strftime(date_format, time.localtime(time.time()))
        source_date = datetime.strptime(source_date, date_format).timestamp()
        target_date = datetime.strptime(target_date, date_format).timestamp()
        compare = source_date - target_date
    except Exception as e:
        zq_log.log_debug('time_str_is_same_day error: %s'%str(e))
    return compare




# 好强大的 pyautogui， 能根据图识别图,然后循环操作
# 可以参考： https://blog.csdn.net/yzy_1996/article/details/85244714
# 可以考虑 算手机分辨率，然后实现手机点击
# import pyautogui
import os, time
# import jieba
from PIL import Image
# from appium import webadb


#region AI自动化相关
# def cv_img_contain_click_pos(img1, img2, isWin, isMultiTouch, click_num=1):
#     ''' 
#     点击img2在img1的位置
#     :param isWin: 电脑是分辨率，手机需要换算分辨率和屏幕大小
#     :param isMultiTouch: 是否是多点触控（所以满足的点，都点击）
#     :param click_num: 点击次数
#     :return {state:是否点击成功,msg:消息提示}
#     '''
#     resultData = {'state': False, 'msg':''}
#     pos_list = cv_img_contain_img_points(img1, img2)
#     if not pos_list or len(pos_list)==0:
#         resultData['msg'] = '没有匹配到对应图'
#     else:
#         for item in pos_list[:]:
#             if isWin:
#                 pyautogui.moveTo(item[0],item[1])
#                 pyautogui.click(clicks=click_num, interval=0.2)
#             else:
#                 for clickIndex in range(0, click_num):
#                     self.adbService.Click(item[0],item[1])
#                     time.sleep(0.2)
#             if not isMultiTouch:
#                 break
#         resultData['state'] = True
#     return resultData

#endregion



import os, hashlib
from natsort import natsorted
import shutil
import zq_log

# region 文件处理
def file_append(filePath, content, isEnter = False):
    ''' 文件追加 '''
    currDirName = os.path.dirname(filePath)
    if not os.path.exists(currDirName):
        os.makedirs(currDirName)
    fd = os.open(filePath,os.O_RDWR|os.O_APPEND|os.O_CREAT)
    os.write(fd,bytes(content + ('\n' if isEnter else ''), encoding='utf-8'))
    os.close(fd)

def file_write(filePath, content, isEnter = False, isCover=True):
    ''' 文件写入 '''
    currDirName = os.path.dirname(filePath)
    if not os.path.exists(currDirName):
        os.makedirs(currDirName)
    if os.path.exists(filePath) and isCover:
        os.remove(filePath)
    fd = os.open(filePath,os.O_RDWR|os.O_CREAT)
    os.write(fd,bytes(content + ('\n' if isEnter else ''), encoding='utf-8'))
    os.close(fd)

def file_read(filePath, encoding = 'utf-8'):
    ''' 文件读取 '''
    configFs = open(filePath,'r+',encoding = encoding)
    configContent = configFs.read()
    return configContent

def file_exists(filePath):
    ''' 文件是否存在 '''
    return os.path.exists(filePath)

def file_copy(sourceFile, targetFile, isAutodir_create = True):
    ''' 文件复制 '''
    try:
        dirName = os.path.dirname(targetFile)
        if not os.path.exists(dirName) and isAutodir_create:
            os.makedirs(dirName)
        shutil.copyfile(sourceFile, targetFile)
        return True
    except IOError as e:
        zq_log.log_debug('file_copy Error %s'%(str(e)))
        return False

def file_md5(filePath):
    ''' 文件获取MD5 '''
    if not os.path.isfile(filePath):
        return None
    myhash = hashlib.md5()
    f = open(filePath,'rb')
    while True:
        b = f.read(8096)
        if not b :
            break
        myhash.update(b)
    f.close()
    return myhash.hexdigest()

def file_sort(fileList):
    ''' 文件排序 '''
    return natsorted(fileList)

def file_get_curr_dir(path):
    ''' 获取文件所在路径 '''
    return os.path.dirname(os.path.realpath(__file__)).replace('\\','/')

def file_size(path):
    ''' 文件大小字节数 '''
    return os.path.getsize(path)

def file_split(path):
    '''
    分割文件路径和文件名和后缀
    :return path, file_name, suffix
    '''
    path, file_name = os.path.split(path)  # 除去文件名以外的path，文件名
    suffix = file_name.split('.')[-1]  # 文件后缀名
    return path, file_name, suffix

#endregion

#region 文件夹操作
def dir_copy(sourceDir, targetDir):
    ''' 文件夹复制 '''
    zq_log.log_debug('当前处理文件夹 %s -> %s' %(sourceDir, targetDir))
    for f in os.listdir(sourceDir):
        sourceF = os.path.join(sourceDir, f)
        targetF = os.path.join(targetDir, f)
        if os.path.isfile(sourceF):
            #创建目录
            if not os.path.exists(targetDir):
                os.makedirs(targetDir)
            #文件不存在，或者存在但是大小不同，覆盖
            if not os.path.exists(targetF) or (os.path.exists(targetF) and (file_md5(targetF) != file_md5(sourceF))):
                #2进制文件
                open(targetF, 'wb').write(open(sourceF, 'rb').read())
                zq_log.log_debug (u'%s 复制完毕' %(targetF))
            else:
                zq_log.log_debug (u'%s 已存在，不重复复制' %(targetF))
        if os.path.isdir(sourceF):
            dir_copy(sourceF, targetF)

def dir_get_dirs(dirPath, isCurrDir, excludeList = []):
    ''' 文件夹获取所有文件夹 '''
    dirsTemp = []
    for root, dirs, files in os.walk(dirPath):
        for dir in dirs:
            isAppend = False
            if isCurrDir:
                if root == dirPath:
                    isAppend = True
            else:
                isAppend = True
            if isAppend and not dir in excludeList:
                dirsTemp.append(os.path.join(root, dir))
        if isCurrDir and root == dirPath:
            break
    return dirsTemp

def dir_get_files(dirPath, isCurrDir, excludeList = [], suffix_names = []):
    ''' 文件夹获取所有文件 '''
    filesTemp = []
    for root, dirs, files in os.walk(dirPath):
        for filePath in files:
            isAppend = False
            if isCurrDir:
                if root == dirPath:
                    isAppend = True
            else:
                isAppend = True
            if isAppend and not filePath in excludeList and (len(suffix_names) == 0 or filePath.split('.')[-1] in suffix_names):
                filesTemp.append(os.path.join(root, filePath))
        if isCurrDir and root == dirPath:
            break
    return filesTemp

def dir_auto_create(*dirs):
    ''' 文件夹自动创建 '''
    try:
        for dir in dirs:
            if dir and not os.path.exists(dir):
                os.makedirs(dir)
        return dirs
    except Exception as e:
        zq_log.log_debug('dir_auto_create error: %s'%str(e))
    return None

def dir_auto_create_dir_by_file(*files):
    ''' 文件夹根据文件创建 '''
    try:
        dirs = [os.path.dirname(file) for file in files]
        return dir_auto_create(*dirs)
    except Exception as e:
        zq_log.log_debug('dir_auto_create_dir_by_file error: %s'%str(e))
    return None

def dir_auto_path_merage(*dirs_or_files):
    ''' 文件夹及文件路径合并 '''
    dirs_or_files = [item.replace('\\','/') for item in dirs_or_files]
    for index in range(len(dirs_or_files)):
        if index != len(dirs_or_files)-1:
            if dirs_or_files[index+1].startswith('./'):
                dirs_or_files[index+1] = dirs_or_files[index+1][2:]
            elif dirs_or_files[index+1][1] == ':':
                dirs_or_files[index+1] = dirs_or_files[index+1][3:-1]

            if dirs_or_files[index].endswith('/') and dirs_or_files[index+1].startswith('/'):
                dirs_or_files[index+1] = dirs_or_files[index+1][1:]
            elif not dirs_or_files[index].endswith('/') and not dirs_or_files[index+1].startswith('/'):
                dirs_or_files[index] = '%s/'%dirs_or_files[index]
    return ''.join(dirs_or_files)

def dir_remove(dir_path, contain_parent_dir):
    '''
    删除文件夹
    :param dir_path: 文件夹路径
    :param contain_parent_dir: 是否包含传入的目录一起删除
    '''
    if contain_parent_dir:
        if os.path.exists(dir_path):
            shutil.rmtree(dir_path, ignore_errors=True)
            # os.system('rm -rf '%s''%os.path.abspath(dir_path))
    else:
        ls = os.listdir(dir_path)
        for i in ls:
            c_path = os.path.join(dir_path, i)
            if os.path.isdir(c_path):
                dir_remove(c_path, True)
            else:
                os.remove(c_path)
#endregion


#region 邮件
import smtplib
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.header import Header

import zq_log

def dosc():
    zq_log.log_debug('email 邮件收发系统')

class email_class():
    def __init__(self, host='smtp.qq.com', user='1620829248@qq.com', pw='rdtbkktznzpkddec'): #vsjgukkyzjjcfacd
        self.mail_host= host    #设置服务器
        self.mail_user= user    #用户名
        self.mail_pass= pw      #口令 密码
        self.sender = user
    
    def SendMail(self, userName, title, content, *receivers):
        '''
        发送邮件
        :param username: 用户名
        :param title: 标题
        :param content: 内容
        :param receivers: 接受方邮箱地址
        '''
        msgRoot = MIMEMultipart('related')
        msgRoot['From'] = Header(userName, 'utf-8')
        msgRoot['To'] =  Header('用户', 'utf-8')
        subject = title #'Python SMTP 邮件测试'
        msgRoot['Subject'] = Header(subject, 'utf-8')
        
        msgAlternative = MIMEMultipart('alternative')
        msgRoot.attach(msgAlternative)
        
        
        mail_msg = '''
        <p>Python 邮件发送测试...</p>
        <p><a href='http://vip.mayishidai.cn'>菜鸟教程链接</a></p>
        <p>图片演示：</p>
        <p><img src='cid:image1'></p>
        '''
        # msgAlternative.attach(MIMEText(mail_msg, 'html', 'utf-8'))
        msgAlternative.attach(MIMEText(content, 'html', 'utf-8'))
        
        '''
        fp = open('./image/test.jpg', 'rb')
        msgImage = MIMEImage(fp.read())
        fp.close()
        
        # 定义图片 ID，在 HTML 文本中引用
        msgImage.add_header('Content-ID', '<image1>')
        msgRoot.attach(msgImage)
        '''

        reTitle = title
        if len(title + content) < 256 :
        	reTitle = title + content
        try:
            smtpObj = smtplib.SMTP() 
            smtpObj.connect(self.mail_host, 25)    # 25 为 SMTP 端口号
            smtpObj.login(self.mail_user,self.mail_pass)
            smtpObj.sendmail(self.sender, receivers, msgRoot.as_string())
            zq_log.log_debug('邮件【success】：%s \n %s'%(reTitle, content))
            return True
        except smtplib.SMTPException as e:
            zq_log.log_debug('邮件【faile】：%s \n %s \n %s'%(reTitle, content, str(e)))
            return False

if __name__ == '__main__':
    email = email_class()
    content = '''
        <p>Python 邮件发送测试...</p>
        <p><a href='http://vip.mayishidai.cn'>菜鸟教程链接</a></p>
        <p>图片演示：</p>
        <p><img src='cid:image1'></p>
        '''
    email.SendMail('用户','嘿嘿嘿',content,'877665041@qq.com')

#endregion


import cv2
import numpy
from moviepy.editor import *
from natsort import natsorted
from PIL import Image
import os, glob, hashlib, shutil
# import zq_cv_img
from moviepy.editor import CompositeAudioClip, VideoFileClip, AudioFileClip, CompositeVideoClip

def _result_param(status, path, video, obj, msg):
    '''
    常规返回数据类型
    :param status: 状态
    :param path: 路径
    :param video: 视频
    :param obj: 对象
    :param msg: 消息
    :return 
    '''
    return {'status':status, 'path': path, 'video': video, 'obj': obj, 'msg': msg}

def _video_path_handle(video_path, save_path):
    '''
    视频路径处理，重名那种
    :param video_path: 视频地址
    :param save_path: 保存地址
    :return 类型: 描述
    '''
    tempPath = video_path
    if video_path == save_path:
        tempFileName = "tempVideo"
        [dirName,fileName]=os.path.split(video_path)
        fileName = "-y.mp4"
        if dirName == "":
            tempPath = "{}_{}".format(tempFileName,fileName)
        else:
            tempPath = "{}/{}_{}".format(dirName,tempFileName,fileName)
        shutil.copyfile(video_path,tempPath)
    return tempPath, save_path

def cv_video_su_miao(video_path):
    '''
    视频转素描
    :param video_path: 视频路径
    '''
    vc = cv2.VideoCapture(video_path)
    c = 0
    if vc.isOpened():
        rval,frame = vc.read()
        height,width = frame.shape[0],frame.shape[1]
        print(height, width)
    else:
        rval = False
        height,width = 960,1200

    # jpg_list = [os.path.join('Pic_Directory/',i) for i in os.listdir('Pic_Directory') if i.endswith('.jpg')]

    fps = 24 # 视频帧率
    video_path1 = './text.mp4'
    video_writer = cv2.VideoWriter(video_path1,cv2.VideoWriter_fourcc(*'mp4v'),fps,(width*2,height))

    while rval:
        rval,frame = vc.read()# 读取视频帧
        img = convert_jpg(Image.fromarray(frame))
        frame_converted = numpy.array(img)

        # 转化为三通道
        image = numpy.expand_dims(frame_converted,axis = 2)
        result_arr = numpy.concatenate((image,image,image),axis = -1)
        result_arr = numpy.hstack((frame,result_arr))

        video_writer.write(result_arr)
        print('Sucessfully Conveted---------{}'.format(c))
        c = c + 1
        if c >= 60:
            break
    video_writer.release()

def cv_video_subclip(video_path, save_path, start_pos, end_pos, is_relative_end_pos = False):
    '''
    视频截取
    :param video_path: 视频地址
    :return list: 视频列表
    '''
    tempPath, save_path = _video_path_handle(video_path, save_path)
    video = VideoFileClip(video_path)
    if is_relative_end_pos:
        # 剪辑视频，从x秒开始到视频结尾前x秒
        video = video.subclip(start_pos, video.duration-end_pos)
    else:
        # 剪辑视频，截取视频前x秒
        video = video.subclip(start_pos, end_pos)
    video.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, video, None, '')

def cv_video_merge_all_video_by_list(video_list, save_path, suffix_name='.mp4'):
    '''
    合成列表所有的视频
    :param video_list: 视频列表
    :param save_path: 保存路径
    '''
    # 定义一个数组
    target_list= []
    # 遍历所有文件
    for file_path in video_list:
        # 如果后缀名为 .mp4
        if os.path.splitext(file_path)[1] == suffix_name:
            # 载入视频
            video = VideoFileClip(file_path)
            # 添加到数组
            target_list.append(video)
    if len(target_list) == 0:
        return _result_param(False, save_path, None, None, '文件列表是空的')
    try:
        # 拼接视频
        final_clip = concatenate_videoclips(target_list)
        # 生成目标视频文件
        final_clip.to_videofile(save_path, fps=25, remove_temp=True)
        return _result_param(True, save_path, final_clip, None, '')
    except BaseException as e:
        return _result_param(False, save_path, None, None, str(e))

def cv_video_merge_all_video_by_dir(dir_path, save_path, suffix_name='.mp4'):
    '''
    合成文件夹下所有的视频
    :param dir_path: 视频文件夹路径
    :param save_path: 保存路径
    :return
    '''
    video_list = []
    # 访问 video 文件夹 (假设视频都放在这里面)
    for root, dirs, files in os.walk(dir_path):
        # 按文件名排序
        # files.sort() # 原 1.mp4, 2.mp4, 10.mp4.  排序后  1.mp4, 10.mp4, 2.mp4.
        files = natsorted(files) # 原 1.mp4, 2.mp4, 10.mp4. 排序后  1.mp4, 2.mp4, 10.mp4.
        # 遍历所有文件
        for file in files:
            # 如果后缀名为 .mp4
            if os.path.splitext(file)[1] == suffix_name:
                # 拼接成完整路径
                filePath = os.path.join(root, file)
                video_list.append(filePath)
    if len(video_list) == 0:
        return _result_param(False, save_path, None, None, '列表为空')
    return cv_video_merge_all_video_by_list(video_list, save_path, suffix_name)

def cv_video_crop(video_path, save_path, x_center, y_center, width, height):
    '''
    视频播放区域裁剪
    :param video_path: 视频路径
    :param save_path: 保存地址
    :param x_center: x中心
    :param y_center: y中心
    :param width: 宽度
    :param height: 高度
    :return 
    '''
    video = VideoFileClip(video_path)
    final_clip = video.crop(x_center=x_center, y_center=y_center, width=width, height=height)
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')

def cv_video_resize(video_path, save_path, width, height):
    '''
    改变视频分辨率,可能花屏(视频画质不变)
    :param video_path: 视频路径
    :param save_path: 保存地址
    :param width: 宽度
    :param height: 高度
    :return 
    '''
    video = VideoFileClip(video_path)
    final_clip = video.resize(newsize=(width, height))
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')


def cv_video_resize_crop(video_path, save_path, x1=0,x2=0,y1=0,y2=0,width=0,height=0,x_center=0,y_center=0):
    '''
    改变视频分辨率,可能花屏(部分画质)
    https://www.sohu.com/a/383491741_797291
    :param video_path: 视频路径
    :param save_path: 保存地址
    :param x1: 左上角
    :param x2: 左上角
    :param y1: 右下角
    :param y2: 右下角
    :param width: 宽度
    :param height: 高度
    :param x_center: x中心点
    :param y_center: y中心点
    :return 
    '''
    video = VideoFileClip(video_path)
    final_clip = video.crop(x1=x1,x2=x2,y1=y1,y2=y2,width=width,height=height,x_center=x_center,y_center=y_center)
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')

def cv_video_rotate(video_path, save_path, ang=0):
    '''
    视频旋转
    :param video_path: 视频路径
    :param save_path: 保存地址
    :param ang: 角度
    :return 
    '''
    video = VideoFileClip(video_path)
    final_clip = video.rotate(ang)
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')

def cv_video_images_clip_by_list(pic_list, save_path, fps=25, suffix_name='.jpg'):
    '''
    把图片列表合成视频
    :param pic_list: 图片列表
    :param save_path: 保存路径
    '''
    target_list= []
    for pic_path in pic_list:
        if os.path.splitext(pic_path)[1] == suffix_name:
            # 添加到数组
            target_list.append(pic_path)
    if len(target_list)==0:
        return _result_param(False, save_path, None, None, '列表为空')
    try:
        final_clip = ImageSequenceClip(pic_list, fps=fps)
        final_clip.to_videofile(save_path, fps=fps, remove_temp=True)
    except BaseException as e:
        print("很大可能是图片不一样大，可以调用zq_cv_img.cv_img_resize_by_dir 或者 images_resize_by_list 统一设置图片大小")
        return _result_param(False, save_path, None, None, str(e))
    return _result_param(True, save_path, final_clip, None, '')

def cv_video_images_clip_by_dir(dir_path, save_path, fps=25, suffix_name='.jpg'):
    '''
    把文件夹下所有图片合成视频
    :param dir_path: 文件夹路径
    :param save_path: 保存地址
    :return 
    '''
    pic_list = []
    for root, dirs, files in os.walk(dir_path):
        # 按文件名排序
        # files.sort() # 原 1.mp4, 2.mp4, 10.mp4.  排序后  1.mp4, 10.mp4, 2.mp4.
        files = natsorted(files) # 原 1.mp4, 2.mp4, 10.mp4. 排序后  1.mp4, 2.mp4, 10.mp4.
        # 遍历所有文件
        for file in files:
            # 如果后缀名为 .mp4
            if os.path.splitext(file)[1] == suffix_name:
                # 拼接成完整路径
                filePath = os.path.join(root, file)
                pic_list.append(filePath)
    if len(pic_list) == 0:
        return _result_param(False, save_path, None, None, '列表为空')
    return cv_video_images_clip_by_list(pic_list, save_path, fps, suffix_name)

def cv_video_merge_video_to_one_canvas(video_list, save_path):
    '''
    把两个视频放在一个画面上同时播放(左右)
    :param video_list: 视频列表
    :param save_path: 保存路径
    '''
    video1 = VideoFileClip(video_list[0])
    video2 = VideoFileClip(video_list[1])
    # 注意：.set_position([x1, y1])中的x1，y1为视频左上角的坐标
    target1 = video1.set_position([0, 0])
    target2 = video2.set_position([video1.w, 0])
    final_clip = CompositeVideoClip([target1, target2], size=(video1.w+video2.w, video1.h))
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')

def video_merge_video_to_one_canvas_by_pos(video_list, save_path, position):
    '''
    把两个视频放在一个画面上同时播放(自定义位置)
    :param video_list: 视频列表
    :param save_path: 保存路径
    :param position: 位置
    '''
    video1 = VideoFileClip(video_list[0])
    video2 = VideoFileClip(video_list[1])
    # 注意：.set_position([x1, y1])中的x1，y1为视频左上角的坐标
    target1 = video1.set_position([0, 0])
    target2 = video2.set_position(position)
    final_clip = CompositeVideoClip([target1, target2], size=(video1.w+video2.w, video1.h))
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')

def cv_video_merge_list_all_video(video_path_list, save_path):
    '''
    视频列表合并为一个视频
    :param video_path_list: 视频路径列表
    :param save_path: 保存地址
    '''
    clips = []
    clip = None
    for video_path in video_path_list:
        try:
            clip = VideoFileClip(video_path)
        except BaseException as e:
            print("cv_video_merge_list_all_video Error " + str(e))
            clip = None
        finally:
            if clip != None:
                clips.append(clip)
    if len(clips) == 0 :
        return _result_param(False, save_path, None, None, '列表为空')
    
    final_clip = concatenate_videoclips(clips)
    # mp4文件默认用libx264编码， 比特率单位bps
    # final_clip.write_videofile(save_path, codec="libx264", bitrate="10000000") 
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')

def cv_video_change_md5(video_path, save_path):
    '''
    改变视频MD5，路径需要带后缀 VideoFileClipPath
    :param video_path: 视频地址
    :param save_path: 保存地址
    '''
    tempPath, save_path = _video_path_handle(video_path, save_path)
    return cv_video_merge_list_all_video([tempPath], save_path)

def cv_video_add_log(video_path, save_path, logoPngPath = None, logoTextStr = None, color='white', align='center', fontsize = 25, left=0, top=0, right=0, bottom=0, opacity=0, pos=('center','center'), fontPath=None):
    '''
    视频添加LOG  VideoClipAddLogoPath
    '''
    tempPath, save_path = _video_path_handle(video_path, save_path)
    if logoPngPath == None and logoTextStr == None:
        return _result_param(False, save_path, None, None, '没有设置水印')

    clipList = []        
    video = VideoFileClip(tempPath)
    clipList.append(video)
    logoPngClip = None
    logoTextClip = None
    if logoPngPath != None:
        logoPngClip = (ImageClip(logoPngPath)
                .set_duration(video.duration)  # 水印持续时间
                .resize(height=50)  # 水印的高度，会等比缩放
                .margin(left=left, top=top, right=right, bottom=bottom, opacity = opacity)# 水印边距和透明度
                .set_pos(pos))  # 水印的位置
        clipList.append(logoPngClip)

    if logoTextStr != None:
        FONT_URL = fontPath or r'C:/Windows/Fonts/simsun.ttc'  #r'C:\Windows\Fonts\simhei.ttf'    #C:\Windows\Fonts\STXINGKA.TTF'
        if not os.path.exists(FONT_URL):
            print("字体文件不存在 %s"%FONT_URL)
            return False
        logoTextClip = (TextClip(txt=logoTextStr, color=color, size=(640, 480), method='caption',
                align=align, fontsize=fontsize, font=FONT_URL)
                .set_duration(video.duration)
                .margin(left=left, top=top, right=right, bottom=bottom, opacity = opacity)
                .set_pos(pos))  # 水印的位置
        clipList.append(logoTextClip)

    if len(clipList) == 0:
        return _result_param(False, save_path, None, None, '列表为空')
    final_clip = CompositeVideoClip(clipList)
    # mp4文件默认用libx264编码， 比特率单位bps
    # final_clip.write_videofile(save_path, codec="libx264", bitrate="10000000") 
    final_clip.to_videofile(save_path, fps=25, remove_temp=True)
    return _result_param(True, save_path, final_clip, None, '')

def cv_video_add_cover(pic_list, video_path, save_path, cover_video_path=None, fps=25, suffix_name='.jpg'):
    '''
    视频添加封面
    '''
    video = VideoFileClip(video_path)
    if not cover_video_path:
        cover_video_path = save_path
    for item in pic_list:
        img = Image.open(item)
        if video.size != list(img.size):
            try:
                new_img = img.resize((video.w, video.h), Image.BILINEAR)
                new_img.save(item)
            except Exception as e:
                print(e)
        
    result = cv_video_images_clip_by_list(pic_list, cover_video_path, fps, suffix_name)
    if result and result['status']:
        return cv_video_merge_all_video_by_list([cover_video_path, video_path], save_path)
    return _result_param(False, save_path, None, result, result['msg'])

def cv_video_add_cover_by_num(pic, video_path, save_path, cover_video_path=None, picNum=25, fps=25, suffix_name='.jpg'):
    '''
    视频添加封面
    '''
    picList = []
    for item in range(0, picNum):
        picList.append(pic)
    else:
        picList.append(pic)
    return cv_video_add_cover(picList, video_path, save_path, cover_video_path, fps, suffix_name)

def cv_video_get_audio(video_path, save_audio_path):
    '''
    提取视频中的音频
    '''
    video = VideoFileClip(video_path)
    audio = video.audio
    audio.write_audiofile(save_audio_path)
    return _result_param(True, save_audio_path, video, audio, '')

def cv_video_add_audio_by_video(video_path_source, video_path_target, out_video_path):
    '''
    为视频添加另一个视频的音频  source 声音添加到 target
    '''
    video_source = VideoFileClip(video_path_source)
    video_target = VideoFileClip(video_path_target)
    audio = video_source.audio
    videoclip = video_target.set_audio(audio)
    videoclip.write_videofile(out_video_path, audio_codec="aac")
    return _result_param(True, out_video_path, videoclip, None, '')

def cv_video_add_audio_by_audio(video_path, audio_path, out_video_path):
    '''
    为视频添加的音频
    '''
    video = VideoFileClip(video_path)
    audio = AudioFileClip(audio_path).subclip(0, video.duration)
    videoclip = video.set_audio(audio)
    videoclip.write_videofile(out_video_path, audio_codec="aac")
    return _result_param(True, out_video_path, videoclip, None, '')

def cv_video_imgs_to_video(img_list, output_video_file, frame_rate):
    '''
    将图片列表组合成视频
    '''
    # 拿一张图片确认宽高
    img0 = cv2.imread(img_list[0])
    # print(img0)
    height, width, layers = img0.shape
    # 视频保存初始化 VideoWriter
    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    videowriter = cv2.VideoWriter(output_video_file, fourcc, frame_rate, (width, height))
    # 核心，保存的东西
    for img_path in img_list:
        img = cv2.imread(img_path)
        videowriter.write(img)
    videowriter.release()
    cv2.destroyAllWindows()
    print('Success save %s!' % output_video_file)

def cv_video_add_audio(video_path, audio_path, out_video_path):
    '''
    为视频添加音频
    '''
    video = VideoFileClip(video_path)
    audio = AudioFileClip(audio_path)
    videos = video.set_audio(audio) #音频文件
    videos.write_videofile(out_video_path,  audio_codec='aac') #保存合成视频，注意加上参数audio_codec='aac'，否则音频无声音
    videos.close()

def cv_video_split_clip(video_path, out_video_path, sub:list):
    '''
    视频分割片段
    '''
    video = VideoFileClip(video_path).subclip(sub[0], sub[1])
    video.write_videofile(out_video_path,  audio_codec='aac') #保存合成视频，注意加上参数audio_codec='aac'，否则音频无声音
    video.close()

def cv_video_to_image(video_path, out_dir, num = -1):
    '''
    将视频分割为图片
    '''
    vid_cap = cv2.VideoCapture(video_path)    #传入视频的路径
    success, image = vid_cap.read()
    count = 0
    while success:
        vid_cap.set(cv2.CAP_PROP_POS_MSEC, 0.5 * 1000 * count)   #截取图片的方法  此处是0.5秒截取一个  可以改变参数设置截取间隔的时间
        if not os.path.exists(out_dir):   #创建每一个视频存储图片对应的文件夹
            os.makedirs(out_dir)
        cv2.imwrite(out_dir+'/' + str(count) + '.jpg', image)       #存储图片的地址 以及对图片的命名
        success, image = vid_cap.read()
        count += 1
        if num != -1 and count >= num:
            break
    vid_cap.release()

def cv_video_merage_video(source_video_path, target_video_path, out_video_path, target_pos=('right', 'bottom'), target_ratio=(1/3, 1/3), target_ratio_fixed:tuple=None,
        target_margin=1, target_margin_color=(255, 255, 255), target_margin_space=(1,1,1,1,0), target_audio=False, fps=24, codec='libx264'):
    '''
    视频合并视频
    '''
    source = VideoFileClip(source_video_path, audio=True)
    w, h = source.size
    if target_ratio_fixed:
        target_ratio = (target_ratio_fixed[0]/w, target_ratio_fixed[1]/h)
    target = (VideoFileClip(target_video_path, audio=target_audio)
        .resize((w * target_ratio[0], h * target_ratio[1]))  # one third of the total screen
        .margin(target_margin, color=target_margin_color)  # white margin
        .margin(left=target_margin_space[0],bottom=target_margin_space[1], right=target_margin_space[2], top=target_margin_space[3], opacity=target_margin_space[4])  # transparent
        .set_pos(target_pos))
    video = CompositeVideoClip([source, target])
    video.write_videofile(out_video_path, fps=fps, codec=codec)
    video.close()



import pytesseract, os, jieba, PIL

def cv_img_contain_text_points(text, img, isRemoveOutBox=True, isShowLog=False):
    '''
    instruction:
    基于tesseract识别出文字(不同语言有不同的文字识别库，此处用的是中文库 chi_sim，主要用于识别中文 )，
    返回文字在屏幕上的坐标点列表。

    usage:
    text = '带查找文字'
    img = '图片地址.png'
    isRemoveOutBox = '结束是否移除数据文件'

    return 匹配到的所有的列表(第一个列表相识度最高)
    '''
    outSuffix = '.box'
    outName='out'
    outFullName='%s%s'%(outName, outSuffix)

    if os.path.exists(img):
        strValue = pytesseract.image_to_string(img, lang='chi_sim')
        if isShowLog:
            print(strValue)
        os.system('tesseract %s %s -l chi_sim makebox'%(img, outName))
        print('输出坐标文件 : %s'%outFullName)
    else:
        print('%s is not exists'%img)

    pos_list = []
    if os.path.exists(outFullName):
        with open(outFullName,'r', encoding='UTF-8') as f:
            for line in f:
                if line.split()[0] in text:
                    pos_list.append(line.split())

    if os.path.exists(outFullName) and isRemoveOutBox:
        os.remove(outFullName)

    return pos_list

def img_contain_text_click_pos(text, img, isWin, isHorizontal=True, isRemoveImg=True, isRemoveOutBox=True, isShowLog=False):
    '''
    点击文字在图片中的位置
    基于tesseract识别出文字(不同语言有不同的文字识别库，此处用的是中文库 chi_sim，主要用于识别中文 )，
    return {state:是否点击成功,msg:消息提示}

    usage:
    text = '带查找文字'
    img = '图片地址.png'
    isWin: 电脑是分辨率，手机需要换算分辨率和屏幕大小
    isRemoveImg = '结束是否移除Img'
    isRemoveOutBox = '结束是否移除数据文件'
    '''

    resultData = {'state': False, 'msg':''}
    pos_list = cv_img_contain_text_points(text, img, isRemoveOutBox)

    if not pos_list or len(pos_list) == 0:
        resultData['msg'] = '没有匹配到对应字'
    else:
        if isShowLog:
            print(pos_list)
        xCenterPointList, yCenterPointList, xCenterPointSum, yCenterPointSum, x, y = [], [], 0, 0, 0, 0
        for item in pos_list[:]:
            tempXCenterPoint = (int(item[1]) + int(item[3]))/2
            xCenterPointList.append(tempXCenterPoint)
            xCenterPointSum = xCenterPointSum + tempXCenterPoint

            tempYCenterPoint = (int(item[2]) + int(item[4]))/2
            yCenterPointList.append(tempYCenterPoint)
            yCenterPointSum = yCenterPointSum + tempYCenterPoint

        x = int(xCenterPointSum/2)
        imgInstance = PIL.Image.open(img)
        height = imgInstance.size[1]    # 获取屏幕高度

        if isHorizontal:
            # 水平，一般认为三个字的中间点Y轴是一样的，所以取一个字的Y轴就可以了
            y = int((height - int(pos_list[0][2])) + (height - int(pos_list[0][4])))/2
        else:
            print('纵向文字的高计算有问题，记录记录，后面修')
            y = int(yCenterPointSum/2)

        if isShowLog:
            print('X--> %s   Y--> %s'%(x, y))
        resultData['state'] = True
    if  os.path.exists(img) and isRemoveImg:
        os.remove(img)
    return resultData

def img_contain_text(img, text, lang='chi_sim', isShowContent=False):
    '''
    图片是否包含文字
    return bool
    '''
    resultStr = pytesseract.image_to_string(img, lang=lang)
    resultStr = resultStr.replace('\n', '').replace(' ', '')
    if isShowContent:
        print(resultStr)
    if type(text) == type([]):
        for item in text[:]:
            if item in resultStr:
                return True
    else:
        if text in resultStr:
            return True
    return False

def img_contain_text_jieba(self, img, text, lang='chi_sim', confidence=None, cut_all=False, isShowContent=False):
    '''
    图片是否包含某一句话（结巴）
    confidence 匹配度
    return 【传入匹配度，返回bool】【不传入匹配度，返回匹配度（0-1）】
    '''
    seg_list = jieba.cut(text)
    if not seg_list:
        return 0
    resultStr = pytesseract.image_to_string(img, lang=lang)
    resultStr = resultStr.replace('\n', '').replace(' ', '')
    if isShowContent:
        print(resultStr)
    used, sum = 1, 1
    for item in seg_list:
        if item[0] in resultStr:
            used += 1
        sum += 1
    confidenceT = used/sum
    if confidence:
        return confidenceT >= confidence
    else:
        return confidenceT


import aircv
import qrcode
import PIL
import numpy
import os
import cv2

import zq_log, zq_file, zq_const

def cv_img_contain_img_points_by_cv2(bg,tp,out=None):
    '''
    滑动模块验证(抖音已验证) 适用于滑动模块
    bg: 背景图片
    tp: 缺口图片
    out:输出图片
    '''
    # 读取背景图片和缺口图片
    bg_img = cv2.imread(bg) # 背景图片
    tp_img = cv2.imread(tp) # 缺口图片
    
    # 识别图片边缘
    bg_edge = cv2.Canny(bg_img, 100, 200)
    tp_edge = cv2.Canny(tp_img, 100, 200)
    
    # 转换图片格式
    bg_pic = cv2.cvtColor(bg_edge, cv2.COLOR_GRAY2RGB)
    tp_pic = cv2.cvtColor(tp_edge, cv2.COLOR_GRAY2RGB)
    
    # 缺口匹配
    res = cv2.matchTemplate(bg_pic, tp_pic, cv2.TM_CCOEFF_NORMED)
    min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res) # 寻找最优匹配
    
    tl = max_loc # 左上角点的坐标
    # 绘制方框
    if out:
        th, tw = tp_pic.shape[:2] 
        br = (tl[0]+tw,tl[1]+th) # 右下角点的坐标
        cv2.rectangle(bg_img, tl, br, (0, 0, 255), 2) # 绘制矩形
        cv2.imwrite(out, bg_img) # 保存在本地
    
    # 返回缺口的X坐标
    return tl[0]

def cv_img_contain_img_points_by_aircv(img1, img2): 
    ''' 
    查找img2在img1的位置（坐标从左上角开始）适用于完全一致
    :param img1 图1
    :param img2 图2
    :return 匹配到的所有的列表(第一个列表相识度最高)
    '''
    imsrc = aircv.imread(img1) # 原始图像
    imsch = aircv.imread(img2) # 带查找的部分
    match_result=aircv.find_all_template(imsrc, imsch,0.80)
    #提取出中心点的横纵坐标
    points=[]
    for i in match_result:
        # [result点坐标，rectangle四个角坐标=>左上坐下右上右下，confidence匹配度=>0.8以上基本就是了]
        points.append((i['result']))
    return points

def cv_img_qr_code_create(codeContent, codeCoreImgPath=None, savePath=None, box_size = 8):
    '''
    生成二维码
    :param codeContent: 内容
    :param codeCoreImgPath: 中心图
    :param savePath: 保存路径
    '''
    qr = qrcode.QRCode(
        version=2,
        error_correction=qrcode.constants.ERROR_CORRECT_H,
        box_size=box_size,
        border=1
    )#各参数具体用法请参考底下链接
    qr.add_data(codeContent)#要显示的内容
    qr.make(fit=True)

    img = qr.make_image()
    img = img.convert('RGBA')
    if codeCoreImgPath:
        icon = PIL.Image.open(codeCoreImgPath)#打开一张图片作为二维码中心图标

        img_w, img_h = img.size
        factor = 4
        size_w = int(img_w / factor)
        size_h = int(img_h / factor)

        icon_w, icon_h = icon.size
        if icon_w > size_w:
            icon_w = size_w
        if icon_h > size_h:
            icon_h = size_h
        icon = icon.resize((icon_w, icon_h), PIL.Image.ANTIALIAS)

        w = int((img_w - icon_w) / 2)
        h = int((img_h - icon_h) / 2)

        img.paste(icon, (w, h))#将图标粘贴到二维码上
    if not savePath:
        savePath = zq_file.dir_auto_path_merage(const.const_caches_dir(), 'qr', 'qrcode.png')
    zq_log.log_debug('二维码保存地址: %s'%savePath)
    img.save(savePath)#保存图片

def cv_img_draw_text(text, sourceImg, targetImg, fontSize=28, fontPath=None, color='#000000', textPosTuple=None, textPosAnchor=None, textPosAnchorOff=None, sizeTuple=None):
    '''
    在图片上绘制文字
    :param text: 内容
    :param sourceImg: 原图
    :param targetImg: 保存图
    :param fontSize: 字体大小
    :param fontPath: 字体路径
    :param color: 字体颜色
    :param textPosTuple 坐标在左上角开始,不填默认最中间
    :param textPosAnchor 坐标锚点，默认最中间 0/默认 中，1 左，2 上，3 右， 4 下
    :param textPosAnchorOff 偏移
    位置：textPosAnchor + textPosAnchorOff适用于添加封面文字, textPosTuple适用于普通文字
    '''
    # 设置字体样式
    font = PIL.ImageFont.truetype(font=fontPath or 'C:/Windows/Fonts/simsun.ttc', size=fontSize)

    # 打开图片
    image = PIL.Image.open(sourceImg)
    draw = PIL.ImageDraw.Draw(image)
    width, height = image.size

    pos = (0,0)
    if textPosTuple:
        pos = textPosTuple
    else:
        if not textPosAnchor or textPosAnchor == 0:
            pos = ((width-fontSize*len(text))/2, (height-fontSize)/2)
        elif not textPosAnchor or textPosAnchor == 1:
            pos = (0, (height-fontSize)/2)
        elif not textPosAnchor or textPosAnchor == 2:
            pos = ((width-fontSize*len(text))/2, 0)
        elif not textPosAnchor or textPosAnchor == 3:
            pos = (width-fontSize*len(text), (height-fontSize)/2)
        elif not textPosAnchor or textPosAnchor == 4:
            pos = ((width-fontSize*len(text))/2, height-fontSize)
        if textPosAnchorOff:
            pos = (pos[0]+textPosAnchorOff[0], pos[1]+textPosAnchorOff[1])

    draw.text(pos, '%s' % text, color, font)
    # 生成图片
    # image.save(targetImg, 'png')
    image.save(targetImg)

    # 压缩图片
    if sizeTuple:
        sImg = PIL.Image.open(targetImg)
        # resize 是裁剪， thumbnail 不会裁剪，是等比缩放
        if sImg.size[0] < sizeTuple[0] and sImg.size[1] < sizeTuple[1]:
            dImg = sImg.resize(sizeTuple, PIL.Image.ANTIALIAS)
            dImg.save(targetImg)
        else:
            sImg.thumbnail(sizeTuple, PIL.Image.ANTIALIAS)
            sImg.save(targetImg)

def cv_img_cut(imgPath, saveDir, saveName, hCutNum, vCutNum=1):
    '''
    图片切图
    :param imgPath:图片地址
    :param saveDir:保存文件夹
    :param saveName:保存名字
    :param hCutNum:横向数量
    :param vCutNum:纵向数量
    '''
    image = PIL.Image.open(imgPath)
    width, height = image.size
    item_width = int(width / hCutNum)
    item_height = int(height / vCutNum)
    box_list = []    
    # (left, upper, right, lower)
    if vCutNum and vCutNum != 1 :
        for j in range(vCutNum):
            for i in range(hCutNum):
                box = (item_width * i, item_height * j, item_width * (i + 1), item_height * (j + 1))
                box_list.append(box)
    else:
        for i in range(hCutNum):
            box = (i*item_width,0,(i+1)*item_width,height)
            box_list.append(box)

    image_list = [image.crop(box) for box in box_list]    
    #保存
    index = 1
    suffixName = '.png'
    for image in image_list:
        image.save('%s%s'%(saveDir, saveName)+str(index) + suffixName)
        index += 1

def cv_img_su_miao(sourceImg, targetImg):
    '''
    将原图转化为 灰度图，并将其像素值放入
    :param sourceImg: 来源图
    :return targetImg: 保存图
    '''
    L = numpy.asarray(sourceImg.convert('L')).astype('float')

    depth = 10.  # (0-100)
    grad = numpy.gradient(L)  # 取图像灰度的梯度值
    grad_x, grad_y = grad  # 分别取横纵图像梯度值
    grad_x = grad_x * depth / 100.
    grad_y = grad_y * depth / 100.
    A = numpy.sqrt(grad_x ** 2 + grad_y ** 2 + 1.)
    uni_x = grad_x / A
    uni_y = grad_y / A
    uni_z = 1. / A

    el = numpy.pi / 2.2  # 光源的俯视角度，弧度值
    az = numpy.pi / 4  # 光源的方位角度，弧度值
    dx = numpy.cos(el) * numpy.cos(az)  # 光源对x轴的影响
    dy = numpy.cos(el) * numpy.sin(az)  # 光源对y轴的影响
    dz = numpy.sin(el)  # 光源对z轴的影响

    gd = 255 * (dx * uni_x + dy * uni_y + dz * uni_z)  # 光源归一化
    gd = gd.clip(0, 255)  # 避免数据越界，将生成的灰度值裁剪至0-255之间

    # gd = numpy.expand_dims(gd,axis =2)
    # gd = numpy.concatenate((gd,gd,gd),axis = -1)
    #
    # gd = numpy.hstack((arr,gd))
    im = PIL.Image.fromarray(gd.astype('uint8'))  # 重构图像
    im.save(targetImg)  # 保存图像
    return im

def cv_img_resize_by_dir(dir_path, save_dir_path, width, height, suffixs_name=['png']):
    '''
    改变某个文件夹下所有指定后缀文件的大小，并保存到指定文件夹下
    :return bool: 是否成功
    '''
    source_list = zq_file.dir_get_files(dir_path, True, suffix_names=suffixs_name)
    for img_path in source_list:
        img_dir = os.path.dirname(img_path)
        if not os.path.exists(img_dir):
            os.makedirs(img_dir)
        img = PIL.Image.open(img_path)
        try:
            new_img = img.resize((width, height), PIL.Image.BILINEAR)
            new_img.save(os.path.join(save_dir_path, os.path.basename(img_path)))
        except Exception as e:
            print(e)
            return False
    return True

def cv_img_add_text(img:str, outImg:str, text, font_size, text_pos:tuple=(0,0), color='black'):
    '''
    图片添加文字
    '''
    bk_img = cv2.imread(img)
    #设置需要显示的字体
    fontpath = 'font/simsun.ttc'
    font = PIL.ImageFont.truetype(fontpath, font_size)
    img_pil = PIL.Image.fromarray(bk_img)
    draw = PIL.ImageDraw.Draw(img_pil)
    #绘制文字信息
    draw.text(text_pos, text, font = font, fill = color)
    bk_img = np.array(img_pil)

    # cv2.imshow('add_text',bk_img)
    # cv2.waitKey()
    cv2.imwrite(outImg,bk_img)
